import os
import os.path

from .fixtures import PACKAGES
from .base import BaseTestCase


class TestCaseCreaterepoComparative_emptyrepo(BaseTestCase):
    """Empty input repository"""

    def test_01_createrepo(self):
        """Repo from empty directory"""
        self.assert_same_results(self.indir)

    def test_02_createrepo_relativepath(self):
        """Repo from empty directory - specified by relative path"""
        self.assert_same_results(os.path.relpath(self.indir))

    def test_03_createrepo_distrotag(self):
        """--distro"""
        self.assert_same_results(self.indir, "--distro DISTRO-TAG")

    def test_04_createrepo_distrotag(self):
        """--distro"""
        self.assert_same_results(self.indir, "--distro CPEID,Footag")

    def test_05_createrepo_distrotag(self):
        """--distro"""
        self.assert_same_results(self.indir, "--distro cpeid,tag_a --distro tag_b")

    def test_06_createrepo_contenttag(self):
        """--content"""
        self.assert_same_results(self.indir, "--content contenttag_a")

    def test_07_createrepo_contenttag(self):
        """--content"""
        self.assert_same_results(self.indir, "--content contenttag_a --content contettag_b")

    def test_08_createrepo_repotag(self):
        """--repo"""
        self.assert_same_results(self.indir, "--repo repotag_a")

    def test_09_createrepo_repotag(self):
        """--repo"""
        self.assert_same_results(self.indir, "--repo repotag_a --repo repotag_b")

    def test_10_createrepo_nodatabase(self):
        """--no-database"""
        self.assert_same_results(self.indir, "--no-database")

    def test_11_createrepo_uniquemdfilenames(self):
        """--unique-md-filenames"""
        self.assert_same_results(self.indir, "--unique-md-filenames")

    def test_12_createrepo_simplemdfilenames(self):
        """--simple-md-filenames"""
        self.assert_same_results(self.indir, "--simple-md-filenames")

    def test_13_createrepo_revision(self):
        """--revision"""
        self.assert_same_results(self.indir, "--revision XYZ")

    def test_14_createrepo_skipsymlinks(self):
        """--skip-symlinks"""
        self.assert_same_results(self.indir, "--skip-symlinks")


class TestCaseCreaterepoComparative_regularrepo(BaseTestCase):
    """Repo with 3 packages"""

    def setup(self):
        self.indir_addpkg(PACKAGES[0])
        self.indir_addpkg(PACKAGES[1])
        self.indir_addpkg(PACKAGES[2])

    def test_01_createrepo(self):
        """Regular createrepo"""
        self.assert_same_results(self.indir)

    def test_02_createrepo_relativepath(self):
        """Regular createrepo"""
        self.assert_same_results(os.path.relpath(self.indir))

    def test_03_createrepo_excludes(self):
        """--excludes * param"""
        self.assert_same_results(self.indir, "--excludes '*'")

    def test_04_createrepo_excludes(self):
        """--excludes"""
        self.assert_same_results(self.indir, "--excludes 'Archer-3.4.5-6.x86_64.rpm'")

    def test_05_createrepo_excludes(self):
        """--excludes"""
        self.assert_same_results(self.indir, "--excludes 'Archer-*.rpm'")

    def test_06_createrepo_skipsymlinks(self):
        """--skip-symlinks"""
        self.assert_same_results(self.indir, "--skip-symlinks")

    def test_07_createrepo_pkglist(self):
        """--pkglist"""
        fn_pkglist = self.indir_mkfile("pkglist", "%s\n" % PACKAGES[0])
        self.assert_same_results(self.indir, "--pkglist %s" % fn_pkglist)

    def test_08_createrepo_pkglist(self):
        """--pkglist"""
        fn_pkglist = self.indir_mkfile("pkglist", "%s\n%s\n" % (PACKAGES[0], PACKAGES[1]))
        self.assert_same_results(self.indir, "--pkglist %s" % fn_pkglist)

    def test_09_createrepo_pkglist(self):
        """--pkglist"""
        fn_pkglist = self.indir_mkfile("pkglist", "%s\n\n%s\n\nfoobar.rpm\n\n" % (PACKAGES[0], PACKAGES[1]))
        self.assert_same_results(self.indir, "--pkglist %s" % fn_pkglist)

class TestCaseCreaterepoComparative_regularrepowithsubdirs(BaseTestCase):
    """Repo with 3 packages, each in its own subdir"""

    def setup(self):
        subdir_a = self.indir_makedirs("a")
        self.copy_pkg(PACKAGES[0], subdir_a)
        subdir_b = self.indir_makedirs("b")
        self.copy_pkg(PACKAGES[1], subdir_b)
        subdir_c = self.indir_makedirs("c")
        self.copy_pkg(PACKAGES[2], subdir_c)

    def test_01_createrepo(self):
        """Regular createrepo"""
        self.assert_same_results(self.indir)

    def test_02_createrepo_skipsymlinks(self):
        """--skip-symlinks"""
        self.assert_same_results(self.indir, "--skip-symlinks")

    def test_03_createrepo_excludes(self):
        """--excludes"""
        self.assert_same_results(self.indir, "--excludes 'Archer-3.4.5-6.x86_64.rpm'")

    def test_04_createrepo_excludes(self):
        """--excludes"""
        self.assert_same_results(self.indir, "--excludes 'Archer-*.rpm'")


class TestCaseCreaterepoComparative_repowithsymlinks(BaseTestCase):
    """Repo with 2 packages and 1 symlink to a package"""

    def setup(self):
        self.indir_addpkg(PACKAGES[0])
        self.indir_addpkg(PACKAGES[1])
        pkg_in_tdir = self.copy_pkg(PACKAGES[2], self.tdir)
        os.symlink(pkg_in_tdir, os.path.join(self.indir, os.path.basename(pkg_in_tdir)))

    def test_01_createrepo(self):
        """Regular createrepo"""
        self.assert_same_results(self.indir)

    def test_02_createrepo_skipsymlinks(self):
        """--skip-symlinks"""
        self.assert_same_results(self.indir, "--skip-symlinks")


class TestCaseCreaterepoComparative_repowithbadpackages(BaseTestCase):
    """Repo with 1 regular package and few broken packages"""

    def setup(self):
        self.indir_addpkg(PACKAGES[0])
        self.indir_makedirs("adirthatlookslike.rpm")
        self.indir_mkfile("emptyfilethatlookslike.rpm")
        self.indir_mkfile("afilethatlookslike.rpm", content="foobar")

    def test_01_createrepo(self):
        """Regular createrepo"""
        self.assert_same_results(self.indir)


class TestCaseCreaterepoComparative_cachedir(BaseTestCase):
    """Repo with 3 packages and cachedir used"""

    def setup(self):
        self.indir_addpkg(PACKAGES[0])
        self.indir_addpkg(PACKAGES[1])
        self.indir_addpkg(PACKAGES[2])

    def test_01_createrepo_owncachedir(self):
        """Each createrepo has its own cachedir"""
        # Gen cache
        self.assert_same_results(self.indir, "--cachedir cache")
        # Run again and use the cache
        _, crres, crcres = self.assert_same_results(self.indir, "--cachedir cache")

        # Compare files in the cache files (they should be identical)
        cr_cache = os.path.join(crres.outdir, "cache")
        crc_cache = os.path.join(crcres.outdir, "cache")
        self.assert_same_dir_content(cr_cache, crc_cache)

    def test_02_createrepo_sharedcachedir(self):
        """Use cache mutually"""
        cache_cr = os.path.abspath(os.path.join(self.indir, "cache_cr"))
        cache_crc = os.path.abspath(os.path.join(self.indir, "cache_crc"))

        # Gen cache by the cr then use it by cr_c
        self.assert_run_cr(self.indir, "--cachedir %s" % cache_cr)
        self.assert_run_cr(self.indir, "--cachedir %s" % cache_cr, c=True)

        # Gen cache by the cr then use it by cr_c
        self.assert_run_cr(self.indir, "--cachedir %s" % cache_crc, c=True)
        self.assert_run_cr(self.indir, "--cachedir %s" % cache_crc)

        # Compare files in the cache files (they should be identical)
        self.assert_same_dir_content(cache_cr, cache_crc)




import os.path

TESTDATADIR = os.path.normpath(os.path.join(__file__, "../../testdata"))
PACKAGESDIR=os.path.join(TESTDATADIR, "packages")

PACKAGES = [
    "Archer-3.4.5-6.x86_64.rpm",
    "fake_bash-1.1.1-1.x86_64.rpm",
    "super_kernel-6.0.1-2.x86_64.rpm",
]


import os
import time
import os.path

from .fixtures import PACKAGES
from .base import BaseTestCase

BASE_XML_PATTERNS_SIMPLE = ["repomd\.xml", "primary\.xml\..+", "filelists\.xml\..+", "other\.xml\..+"]
BASE_XML_PATTERNS_UNIQUE = ["repomd\.xml", ".*-primary\.xml\..+", ".*-filelists\.xml\..+", ".*-other\.xml\..+"]

DBS_PATTERNS = [".*primary\.sqlite\..+", ".*filelists\.sqlite\..+", ".*other\.sqlite\..+"]
DBS_PATTERNS_SIMPLE = ["primary\.sqlite\..+", "filelists\.sqlite\..+", "other\.sqlite\..+"]
DBS_PATTERNS_UNIQUE = [".*-primary\.sqlite\..+", ".*-filelists\.sqlite\..+", ".*-other\.sqlite\..+"]

DBS_PATTERNS_UNIQUE_MD5 = ["[0-9a-z]{32}-primary\.sqlite\..+", ".*-filelists\.sqlite\..+", ".*-other\.sqlite\..+"]

DBS_PATTERNS_SIMPLE_GZ = ["primary\.sqlite\.gz", "filelists\.sqlite\.gz", "other\.sqlite\.gz"]
DBS_PATTERNS_SIMPLE_BZ2 = ["primary\.sqlite\.bz2", "filelists\.sqlite\.bz2", "other\.sqlite\.bz2"]
DBS_PATTERNS_SIMPLE_XZ = ["primary\.sqlite\.xz", "filelists\.sqlite\.xz", "other\.sqlite\.xz"]


class TestCaseSqliterepo_badparams(BaseTestCase):
    """Use case with bad commandline arguments"""

    def setup(self):
        self.indir_addpkg(PACKAGES[0])


class TestCaseSqliterepo(BaseTestCase):
    """Base use cases"""

    def setup(self):
        self.indir_addpkg(PACKAGES[0])

    def test_01_sqliterepo(self):
        """Sqlitedbs already exists, sqliterepo without --force should fail"""
        cr_res = self.run_cr(self.indir, c=True)
        sq_res = self.run_sqlr(cr_res.outdir)
        self.assertTrue(sq_res.rc)
        self.assertTrue("already has sqlitedb present" in sq_res.out)

    def test_02_sqliterepo(self):
        """Sqlitedbs should be created"""
        outdir = self.tdir_makedirs("repository")
        self.assert_run_cr(self.indir, args="--no-database", c=True, outdir=outdir)
        self.assert_run_sqlr(outdir)
        # Check that DBs really exists
        self.assert_repo_files(outdir, DBS_PATTERNS, additional_files_allowed=True)

    def test_03_sqliterepo(self):
        """Sqlitedbs with simple md filenames should be created"""
        outdir = self.tdir_makedirs("repository")
        self.assert_run_cr(self.indir, args="--no-database --simple-md-filenames", c=True, outdir=outdir)
        self.assert_run_sqlr(outdir)
        # Check that DBs really exists
        self.assert_repo_files(outdir, DBS_PATTERNS_SIMPLE, additional_files_allowed=True)

    def test_04_sqliterepo(self):
        """Sqlitedbs with unique md filenames should be created"""
        outdir = self.tdir_makedirs("repository")
        self.assert_run_cr(self.indir, args="--no-database --unique-md-filenames", c=True, outdir=outdir)
        self.assert_run_sqlr(outdir)
        # Check that DBs really exists
        self.assert_repo_files(outdir, DBS_PATTERNS_UNIQUE, additional_files_allowed=True)

    def test_05_sqliterepo(self):
        """--xz is used (old bz2 DBs should be removed)"""
        outdir = self.tdir_makedirs("repository")
        self.assert_run_cr(self.indir, args="--database --simple-md-filenames", c=True, outdir=outdir)
        self.assert_run_sqlr(outdir, args="--force --xz")

        # Check that DBs really exists
        self.assert_repo_files(outdir,
                               DBS_PATTERNS_SIMPLE_XZ+BASE_XML_PATTERNS_SIMPLE,
                               additional_files_allowed=False)

    def test_06_sqliterepo(self):
        """DBs already exists but --force is used sqlitedbs should be created"""
        outdir = self.tdir_makedirs("repository")
        self.assert_run_cr(self.indir, args="--database --simple-md-filenames", c=True, outdir=outdir)
        old_primary_ts = os.path.getmtime(os.path.join(outdir, "repodata", "primary.sqlite.bz2"))

        self.assert_run_sqlr(outdir, args="--force")
        new_primary_ts = os.path.getmtime(os.path.join(outdir, "repodata", "primary.sqlite.bz2"))

        # Check that DBs really exists
        self.assert_repo_files(outdir, DBS_PATTERNS, additional_files_allowed=True)

        # Check that DBs are newer than the old ones
        self.assertTrue(old_primary_ts < new_primary_ts)

    def test_07_sqliterepo(self):
        """--force and --keep-old and --xz are used (old bz2 DBs should be keeped)"""
        outdir = self.tdir_makedirs("repository")
        self.assert_run_cr(self.indir, args="--database --simple-md-filenames", c=True, outdir=outdir)
        self.assert_run_sqlr(outdir, args="--force --keep-old --xz")

        # Check that DBs really exists
        self.assert_repo_files(outdir, DBS_PATTERNS_SIMPLE_BZ2, additional_files_allowed=True)
        self.assert_repo_files(outdir, DBS_PATTERNS_SIMPLE_XZ, additional_files_allowed=True)

    def test_08_sqliterepo(self):
        """--local-sqlite is used and old DBs exists (--force is also used)"""
        outdir = self.tdir_makedirs("repository")
        self.assert_run_cr(self.indir, args="--database --simple-md-filenames", c=True, outdir=outdir)
        old_primary_ts = os.path.getmtime(os.path.join(outdir, "repodata", "primary.sqlite.bz2"))

        self.assert_run_sqlr(outdir, args="--local-sqlite --force")
        new_primary_ts = os.path.getmtime(os.path.join(outdir, "repodata", "primary.sqlite.bz2"))

        # Check that DBs really exists
        expected_files = DBS_PATTERNS_SIMPLE_BZ2 + BASE_XML_PATTERNS_SIMPLE
        self.assert_repo_files(outdir, expected_files, additional_files_allowed=False)

        # Check that DBs are newer than the old ones
        self.assertTrue(old_primary_ts < new_primary_ts)

    def test_09_sqliterepo(self):
        """--local-sqlite is used with --xz and --force and --keep-old"""
        outdir = self.tdir_makedirs("repository")
        self.assert_run_cr(self.indir, args="--database --simple-md-filenames", c=True, outdir=outdir)
        self.assert_run_sqlr(outdir, args="--local-sqlite --force --keep-old --xz")

        # Check that DBs really exists
        self.assert_repo_files(outdir, DBS_PATTERNS_SIMPLE_BZ2, additional_files_allowed=True)
        self.assert_repo_files(outdir, DBS_PATTERNS_SIMPLE_XZ, additional_files_allowed=True)

    def test_10_sqliterepo(self):
        """--compress-type used"""
        outdir = self.tdir_makedirs("repository")
        self.assert_run_cr(self.indir, args="--database --simple-md-filenames", c=True, outdir=outdir)
        self.assert_run_sqlr(outdir, args="--compress-type gz --force")

        # Check that DBs really exists
        expected_files = DBS_PATTERNS_SIMPLE_GZ + BASE_XML_PATTERNS_SIMPLE
        self.assert_repo_files(outdir, expected_files, additional_files_allowed=False)

    def test_11_sqliterepo(self):
        """--checksum used"""
        outdir = self.tdir_makedirs("repository")
        self.assert_run_cr(self.indir, args="--database --unique-md-filenames", c=True, outdir=outdir)
        self.assert_run_sqlr(outdir, args="--checksum md5 --force")

        # Check that DBs really exists
        expected_files = DBS_PATTERNS_UNIQUE_MD5 + BASE_XML_PATTERNS_UNIQUE
        self.assert_repo_files(outdir, expected_files, additional_files_allowed=False)


#!/usr/bin/env python
"""
Convert a single line from test output to run single test command.

E.g:
    "2: test_download_package_via_metalink (tests.test_yum_package_downloading.TestCaseYumPackageDownloading) ... ok"

To:
    tests/python/tests/test_yum_repo_downloading.py:TestCaseYumRepoDownloading.test_download_and_update_repo_01
"""

import os
import sys
import argparse

LIBPATH = "./build/src/python/"
COMMAND = "PYTHONPATH=`readlink -f {libpath}` nosetests{nosever} -s -v {testpath}"
TEST_PATH_PREFIX = "tests/python"

if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='Convert a single line from '\
            'python unittesttest output to run command for this single test')

    parser.add_argument('test_out_line', metavar='TESTOUTPUTLINE',
                        type=str,
                        help='A single line from python unittesttest output')
    parser.add_argument('--python3', action='store_true', help='Python3 version')
    args = parser.parse_args()

    test_out_line = args.test_out_line

    # Remove suffix "... ok" or "... FAIL"
    test_out_line = test_out_line.split(" ... ")[0]

    # Remove prefix "test_number: "
    res = test_out_line = test_out_line.split(": ")
    test_out_line = res[-1]

    # Get test name
    res = test_out_line.split(" ")
    if len(res) != 2:
        print("Bad input line format")
        sys.exit(1)

    test_name, test_out_line = res

    # Get test case
    test_out_line = test_out_line.strip().lstrip("(").rstrip(")")
    res = test_out_line.rsplit(".", 1)
    if len(res) != 2:
        print("Bad input line format")
        sys.exit(1)

    test_out_line, test_case = res

    # Get path
    test_path = test_out_line.replace(".", "/") + ".py"

    full_path = os.path.join(TEST_PATH_PREFIX, test_path)

    nosever = ""
    pythonversion = 2

    if (args.python3):
        nosever = "-3.3"
        pythonversion = 3

    testpath = "{0}:{1}.{2}".format(full_path, test_case, test_name)
    libpath = LIBPATH.format(pythonversion=pythonversion)


    command = COMMAND.format(libpath=libpath, nosever=nosever, testpath=testpath)

    print(command)




import unittest
import shutil
import tempfile
import os.path
import createrepo_c as cr

from .fixtures import *

class TestCaseUpdateCollectionPackage(unittest.TestCase):

    def test_updatecollectionpackage_setters(self):
        pkg = cr.UpdateCollectionPackage()
        self.assertTrue(pkg)

        self.assertEqual(pkg.name, None)
        self.assertEqual(pkg.version, None)
        self.assertEqual(pkg.release, None)
        self.assertEqual(pkg.epoch, None)
        self.assertEqual(pkg.arch, None)
        self.assertEqual(pkg.src, None)
        self.assertEqual(pkg.filename, None)
        self.assertEqual(pkg.sum, None)
        self.assertEqual(pkg.sum_type, 0)
        self.assertEqual(pkg.reboot_suggested, 0)

        pkg.name = "foo"
        pkg.version = "1.2"
        pkg.release = "3"
        pkg.epoch = "0"
        pkg.arch = "x86"
        pkg.src = "foo.src.rpm"
        pkg.filename = "foo.rpm"
        pkg.sum = "abcdef"
        pkg.sum_type = cr.SHA1
        pkg.reboot_suggested = True

        self.assertEqual(pkg.name, "foo")
        self.assertEqual(pkg.version, "1.2")
        self.assertEqual(pkg.release, "3")
        self.assertEqual(pkg.epoch, "0")
        self.assertEqual(pkg.arch, "x86")
        self.assertEqual(pkg.src, "foo.src.rpm")
        self.assertEqual(pkg.filename, "foo.rpm")
        self.assertEqual(pkg.sum, "abcdef")
        self.assertEqual(pkg.sum_type, cr.SHA1)
        self.assertEqual(pkg.reboot_suggested, True)



import unittest
import os.path
import createrepo_c as cr

from .fixtures import *

class TestCaseCompressionWrapper(unittest.TestCase):
    def test_compression_suffix(self):
        self.assertEqual(cr.compression_suffix(cr.AUTO_DETECT_COMPRESSION), None)
        self.assertEqual(cr.compression_suffix(cr.UNKNOWN_COMPRESSION), None)
        self.assertEqual(cr.compression_suffix(cr.NO_COMPRESSION), None)
        self.assertEqual(cr.compression_suffix(123), None)

        self.assertEqual(cr.compression_suffix(cr.GZ), ".gz")
        self.assertEqual(cr.compression_suffix(cr.BZ2), ".bz2")
        self.assertEqual(cr.compression_suffix(cr.XZ), ".xz")
        self.assertEqual(cr.compression_suffix(cr.ZCK), ".zck")

    def test_detect_compression(self):

        # no compression
        path = os.path.join(COMPRESSED_FILES_PATH, "01_plain.txt")
        comtype = cr.detect_compression(path)
        self.assertEqual(comtype, cr.NO_COMPRESSION)

        # gz compression
        path = os.path.join(COMPRESSED_FILES_PATH, "01_plain.txt.gz")
        comtype = cr.detect_compression(path)
        self.assertEqual(comtype, cr.GZ)

        # bz2 compression
        path = os.path.join(COMPRESSED_FILES_PATH, "01_plain.txt.bz2")
        comtype = cr.detect_compression(path)
        self.assertEqual(comtype, cr.BZ2)

        # xz compression
        path = os.path.join(COMPRESSED_FILES_PATH, "01_plain.txt.xz")
        comtype = cr.detect_compression(path)
        self.assertEqual(comtype, cr.XZ)

        # zck compression
        path = os.path.join(COMPRESSED_FILES_PATH, "01_plain.txt.zck")
        comtype = cr.detect_compression(path)
        self.assertEqual(comtype, cr.ZCK)

        # Bad suffix - no compression
        path = os.path.join(COMPRESSED_FILES_PATH, "01_plain.foo0")
        comtype = cr.detect_compression(path)
        self.assertEqual(comtype, cr.NO_COMPRESSION)

        # Bad suffix - gz compression
        path = os.path.join(COMPRESSED_FILES_PATH, "01_plain.foo1")
        comtype = cr.detect_compression(path)
        self.assertEqual(comtype, cr.GZ)

        # Bad suffix - bz2 compression
        path = os.path.join(COMPRESSED_FILES_PATH, "01_plain.foo2")
        comtype = cr.detect_compression(path)
        self.assertEqual(comtype, cr.BZ2)

        # Bad suffix - xz compression
        path = os.path.join(COMPRESSED_FILES_PATH, "01_plain.foo3")
        comtype = cr.detect_compression(path)
        self.assertEqual(comtype, cr.XZ)

        # Disabled because magic module doesn't recognize zchunk files yet
        # Bad suffix - zck compression
        #path = os.path.join(COMPRESSED_FILES_PATH, "01_plain.foo4")
        #comtype = cr.detect_compression(path)
        #self.assertEqual(comtype, cr.ZCK)

    def test_compression_type(self):
        self.assertEqual(cr.compression_type(None), cr.UNKNOWN_COMPRESSION)
        self.assertEqual(cr.compression_type(""), cr.UNKNOWN_COMPRESSION)
        self.assertEqual(cr.compression_type("gz"), cr.GZ)
        self.assertEqual(cr.compression_type("bz2"), cr.BZ2)
        self.assertEqual(cr.compression_type("xz"), cr.XZ)
        self.assertEqual(cr.compression_type("XZ"), cr.XZ)
        self.assertEqual(cr.compression_type("zck"), cr.ZCK)




import unittest
import shutil
import tempfile
import os.path
import createrepo_c as cr

from .fixtures import *

class TestCaseUpdateCollection(unittest.TestCase):

    def test_updatecollection_setters(self):
        col = cr.UpdateCollection()
        self.assertTrue(col)

        self.assertEqual(col.shortname, None)
        self.assertEqual(col.name, None)
        self.assertEqual(col.packages, [])

        module = cr.UpdateCollectionModule()
        module.name = "kangaroo"
        module.stream = "0"
        module.version = 20180730223407
        module.context = "deadbeef"
        module.arch = "noarch"

        pkg = cr.UpdateCollectionPackage()
        pkg.name = "foo"
        pkg.version = "1.2"
        pkg.release = "3"
        pkg.epoch = "0"
        pkg.arch = "x86"
        pkg.src = "foo.src.rpm"
        pkg.filename = "foo.rpm"
        pkg.sum = "abcdef"
        pkg.sum_type = cr.SHA1
        pkg.reboot_suggested = True

        col.shortname = "short name"
        col.name = "long name"
        col.module = module
        col.append(pkg)

        self.assertEqual(col.shortname, "short name")
        self.assertEqual(col.name, "long name")
        self.assertEqual(len(col.packages), 1)

        # Check if the appended module was appended properly
        module = col.module
        self.assertEqual(module.name, "kangaroo")
        self.assertEqual(module.stream, "0")
        self.assertEqual(module.version, 20180730223407)
        self.assertEqual(module.context, "deadbeef")
        self.assertEqual(module.arch, "noarch")

        # Also check if the appended package was appended properly
        pkg = col.packages[0]
        self.assertEqual(pkg.name, "foo")
        self.assertEqual(pkg.version, "1.2")
        self.assertEqual(pkg.release, "3")
        self.assertEqual(pkg.epoch, "0")
        self.assertEqual(pkg.arch, "x86")
        self.assertEqual(pkg.src, "foo.src.rpm")
        self.assertEqual(pkg.filename, "foo.rpm")
        self.assertEqual(pkg.sum, "abcdef")
        self.assertEqual(pkg.sum_type, cr.SHA1)
        self.assertEqual(pkg.reboot_suggested, True)



#!/usr/bin/env python

import sys
import os
import os.path
import hashlib

import createrepo_c as cr

REPO_PATH = "repo/"

class CalculationException(Exception):
    pass

def calculate_contenthash(path):
    if not os.path.isdir(path) or \
       not os.path.isdir(os.path.join(path, "repodata/")):
        raise AttributeError("Not a repo: {0}".format(path))

    repomd_path = os.path.join(path, "repodata/repomd.xml")
    repomd = cr.Repomd(repomd_path)

    primary_path = None
    for rec in repomd.records:
        if rec.type == "primary":
            primary_path = rec.location_href
            break

    if not primary_path:
        raise CalculationException("primary metadata are missing")

    pkgids = []

    def pkgcb(pkg):
        pkgids.append("{0}{1}{2}".format(pkg.pkgId,
                                         pkg.location_href,
                                         pkg.location_base or ''))

    cr.xml_parse_primary(os.path.join(path, primary_path),
                         pkgcb=pkgcb)

    contenthash = hashlib.new("sha256")
    for pkgid in sorted(pkgids):
        contenthash.update(pkgid)
    return contenthash.hexdigest()

if __name__ == "__main__":
    path = REPO_PATH
    if len(sys.argv) == 2:
        path = sys.argv[1]
    print calculate_contenthash(path)



import os
import os.path

from .fixtures import PACKAGES
from .base import BaseTestCase


class TestCaseCreaterepoUpdateComparative_emptyrepo(BaseTestCase):
    """Empty input repository"""

    def test_01_createrepoupdate(self):
        """Repo from empty directory"""
        self.assert_same_results(self.indir)
        self.assert_same_results(self.indir, "--update")

    def test_02_createrepoupdate_double(self):
        """Repo from empty directory"""
        self.assert_same_results(self.indir)
        self.assert_same_results(self.indir, "--update")
        self.assert_same_results(self.indir, "--update")

    def test_03_createrepoupdate_relativepath(self):
        """Repo from empty directory - specified by relative path"""
        self.assert_same_results(os.path.relpath(self.indir))
        self.assert_same_results(os.path.relpath(self.indir), "--update")

    def test_04_createrepoupdate_simplemdfilenames(self):
        """Repo from empty directory - specified by relative path"""
        self.assert_same_results(os.path.relpath(self.indir))
        self.assert_same_results(os.path.relpath(self.indir), "--update --simple-md-filenames")


class TestCaseCreaterepoUpdateComparative_regularrepo(BaseTestCase):
    """Repo with 3 packages"""

    def setup(self):
        self.indir_addpkg(PACKAGES[0])
        self.indir_addpkg(PACKAGES[1])
        self.indir_addpkg(PACKAGES[2])

    def test_01_createrepoupdate(self):
        """Repo from empty directory"""
        self.assert_same_results(self.indir)
        self.assert_same_results(self.indir, "--update")

    def test_02_createrepoupdate_double(self):
        """Repo from empty directory"""
        self.assert_same_results(self.indir)
        self.assert_same_results(self.indir, "--update")
        self.assert_same_results(self.indir, "--update")

    def test_03_createrepoupdate_relativepath(self):
        """Repo from empty directory - specified by relative path"""
        self.assert_same_results(os.path.relpath(self.indir))
        self.assert_same_results(os.path.relpath(self.indir), "--update")

    def test_04_createrepoupdate_simplemdfilenames(self):
        """Repo from empty directory - specified by relative path"""
        self.assert_same_results(os.path.relpath(self.indir))
        self.assert_same_results(os.path.relpath(self.indir), "--update --simple-md-filenames")


class TestCaseCreaterepoUpdateComparative_regularrepoandupdaterepo(BaseTestCase):
    """Repo with 3 packages and repo that will be used as --update-md-path"""

    def setup(self):
        self.indir_addpkg(PACKAGES[0])
        self.indir_addpkg(PACKAGES[1])
        self.indir_addpkg(PACKAGES[2])
        res = self.assert_run_cr(self.indir)
        self.update_md_path = os.path.join(self.tdir, "update_md_path")
        os.rename(res.outdir, self.update_md_path)
        os.rename(res.logfile, os.path.join(self.tdir, "out_createrepo-update_md_path"))

    def test_01_createrepoupdate_updatemdpath(self):
        """Repo from empty directory"""
        self.assert_same_results(self.indir, "--update --update-md-path %s" % self.update_md_path)

    def test_02_createrepoupdate_updatemdpathandupdate(self):
        """Repo from empty directory"""
        self.assert_same_results(self.indir, "--update --update-md-path %s" % self.update_md_path)
        self.assert_same_results(self.indir, "--update")

    def test_03_createrepoupdate_updatemdpathdouble(self):
        """Repo from empty directory"""
        self.assert_same_results(self.indir, "--update --update-md-path %s" % self.update_md_path)
        self.assert_same_results(self.indir, "--update --update-md-path %s" % self.update_md_path)



import unittest
import shutil
import tempfile
import os.path
import createrepo_c as cr

from .fixtures import *

class TestCaseUpdateReference(unittest.TestCase):

    def test_updatereference_setters(self):
        ref = cr.UpdateReference()
        self.assertTrue(ref)

        self.assertEqual(ref.href, None)
        self.assertEqual(ref.id, None)
        self.assertEqual(ref.type, None)
        self.assertEqual(ref.title, None)

        ref.href = "href"
        ref.id = "id"
        ref.type = "type"
        ref.title = "title"

        self.assertEqual(ref.href, "href")
        self.assertEqual(ref.id, "id")
        self.assertEqual(ref.type, "type")
        self.assertEqual(ref.title, "title")



import os.path

TEST_DATA_PATH = os.path.normpath(os.path.join(__file__, "../../../testdata"))

COMPRESSED_FILES_PATH = os.path.join(TEST_DATA_PATH, "compressed_files")
MODIFIED_REPO_FILES_PATH = os.path.join(TEST_DATA_PATH, "modified_repo_files")
PACKAGES_PATH = os.path.join(TEST_DATA_PATH, "packages")
REPOS_PATH = TEST_DATA_PATH
TEST_FILES_PATH = os.path.join(TEST_DATA_PATH, "test_files")

# Modified repo files

PRIMARY_ERROR_00_PATH = os.path.join(MODIFIED_REPO_FILES_PATH,
                        "error_00-primary.xml")
PRIMARY_MULTI_WARN_00_PATH = os.path.join(MODIFIED_REPO_FILES_PATH,
                             "multiple_warnings_00-primary.xml")

FILELISTS_ERROR_00_PATH = os.path.join(MODIFIED_REPO_FILES_PATH,
                          "error_00-filelists.xml")
FILELISTS_MULTI_WARN_00_PATH = os.path.join(MODIFIED_REPO_FILES_PATH,
                               "multiple_warnings_00-filelists.xml")

OTHER_ERROR_00_PATH = os.path.join(MODIFIED_REPO_FILES_PATH,
                      "error_00-other.xml")
OTHER_MULTI_WARN_00_PATH = os.path.join(MODIFIED_REPO_FILES_PATH,
                           "multiple_warnings_00-other.xml")

# Packages

PKG_ARCHER = "Archer-3.4.5-6.x86_64.rpm"
PKG_ARCHER_PATH = os.path.join(PACKAGES_PATH, PKG_ARCHER)

PKG_BALICEK_ISO88591 = "balicek-iso88591-1.1.1-1.x86_64.rpm"
PKG_BALICEK_ISO88591_PATH = os.path.join(PACKAGES_PATH, PKG_BALICEK_ISO88591)

PKG_BALICEK_ISO88592 = "balicek-iso88592-1.1.1-1.x86_64.rpm"
PKG_BALICEK_ISO88592_PATH = os.path.join(PACKAGES_PATH, PKG_BALICEK_ISO88592)

PKG_BALICEK_UTF8 = "balicek-utf8-1.1.1-1.x86_64.rpm"
PKG_BALICEK_UTF8_PATH = os.path.join(PACKAGES_PATH, PKG_BALICEK_UTF8)

PKG_EMPTY = "empty-0-0.x86_64.rpm"
PKG_EMPTY_PATH = os.path.join(PACKAGES_PATH, PKG_EMPTY)

PKG_EMPTY_SRC = "empty-0-0.x86_64.rpm"
PKG_EMPTY_SRC_PATH = os.path.join(PACKAGES_PATH, PKG_EMPTY_SRC)

PKG_FAKE_BASH = "fake_bash-1.1.1-1.x86_64.rpm"
PKG_FAKE_BASH_PATH = os.path.join(PACKAGES_PATH, PKG_FAKE_BASH)

PKG_SUPER_KERNEL = "super_kernel-6.0.1-2.x86_64.rpm"
PKG_SUPER_KERNEL_PATH = os.path.join(PACKAGES_PATH, PKG_SUPER_KERNEL)

# Test repositories

REPO_00_PATH = os.path.join(REPOS_PATH, "repo_00")
REPO_00_REPOMD = os.path.join(REPO_00_PATH, "repodata/repomd.xml")
REPO_00_PRIXML = os.path.join(REPO_00_PATH, "repodata/",
    "1cb61ea996355add02b1426ed4c1780ea75ce0c04c5d1107c025c3fbd7d8bcae-primary.xml.gz")
REPO_00_FILXML = os.path.join(REPO_00_PATH, "repodata/",
    "95a4415d859d7120efb6b3cf964c07bebbff9a5275ca673e6e74a97bcbfb2a5f-filelists.xml.gz")
REPO_00_OTHXML = os.path.join(REPO_00_PATH, "repodata/",
    "ef3e20691954c3d1318ec3071a982da339f4ed76967ded668b795c9e070aaab6-other.xml.gz")
REPO_00_PRIZCK = os.path.join(REPO_00_PATH, "repodata/",
    "e0ac03cd77e95e724dbf90ded0dba664e233315a8940051dd8882c56b9878595-primary.xml.zck")
REPO_00_FILZCK = os.path.join(REPO_00_PATH, "repodata/",
    "2e7db4492173b6c437fd1299dc335e63d09f24cbdadeac5175a61b787c2f7a44-filelists.xml.zck")
REPO_00_OTHZCK = os.path.join(REPO_00_PATH, "repodata/",
    "a939c4765106655c3f7a13fb41d0f239824efa66bcd6c1e6c044a854012bda75-other.xml.zck")

REPO_01_PATH = os.path.join(REPOS_PATH, "repo_01")
REPO_01_REPOMD = os.path.join(REPO_01_PATH, "repodata/repomd.xml")
REPO_01_PRIXML = os.path.join(REPO_01_PATH, "repodata/",
    "6c662d665c24de9a0f62c17d8fa50622307739d7376f0d19097ca96c6d7f5e3e-primary.xml.gz")
REPO_01_FILXML = os.path.join(REPO_01_PATH, "repodata/",
    "c7db035d0e6f1b2e883a7fa3229e2d2be70c05a8b8d2b57dbb5f9c1a67483b6c-filelists.xml.gz")
REPO_01_OTHXML = os.path.join(REPO_01_PATH, "repodata/",
    "b752a73d9efd4006d740f943db5fb7c2dd77a8324bd99da92e86bd55a2c126ef-other.xml.gz")

REPO_02_PATH = os.path.join(REPOS_PATH, "repo_02")
REPO_02_REPOMD = os.path.join(REPO_02_PATH, "repodata/repomd.xml")
REPO_02_PRIXML = os.path.join(REPO_02_PATH, "repodata/",
    "bcde64b04916a2a72fdc257d61bc922c70b3d58e953499180585f7a360ce86cf-primary.xml.gz")
REPO_02_FILXML = os.path.join(REPO_02_PATH, "repodata/",
    "3b7e6ecd01af9cb674aff6458186911d7081bb5676d5562a21a963afc8a8bcc7-filelists.xml.gz")
REPO_02_OTHXML = os.path.join(REPO_02_PATH, "repodata/",
    "ab5d3edeea50f9b4ec5ee13e4d25c147e318e3a433dbabc94d3461f58ac28255-other.xml.gz")

REPO_WITH_ADDITIONAL_METADATA = os.path.join(REPOS_PATH, "repo_with_additional_metadata")

# Test files

FILE_BINARY = "binary_file"
FILE_BINARY_PATH = os.path.join(TEST_FILES_PATH, FILE_BINARY)
FILE_TEXT = "text_file"
FILE_TEXT = os.path.join(TEST_FILES_PATH, FILE_TEXT)
FILE_TEXT_SHA256SUM = "2f395bdfa2750978965e4781ddf224c89646c7d7a1569b7ebb023b170f7bd8bb"
FILE_TEXT_GZ = FILE_TEXT+".gz"
FILE_EMPTY = "empty_file"
FILE_EMPTY = os.path.join(TEST_FILES_PATH, FILE_EMPTY)




#!/usr/bin/python

import sys
import re
import datetime
from optparse import OptionParser


class Info(object):
    def __init__(self, name, summary=None, description=None, synopsis=None, copyright=None, options=None):
        self.name = name
        self.summary = summary
        self.description = description
        self.synopsis = synopsis
        self.copyright = copyright
        self.options = options  # list of dictionaries with keys ["long_name", "short_name", "description", "arg_description"]

    def gen_rst(self):
        rst  = ".. -*- coding: utf-8 -*-\n\n"
        rst += "%s\n" % ("=" * len(self.name),)
        rst += "%s\n" % self.name
        rst += "%s\n\n" % ("=" * len(self.name),)

        # Add summary
        if self.summary:
            rst += "%s\n" % ("-" * len(self.summary))
            rst += "%s\n" % self.summary
            rst += "%s\n" % ("-" * len(self.summary))
            rst += "\n"

        # Add copyright
        if self.copyright:
            rst += ":Copyright: %s\n" % self.copyright

        # Add date
        rst += ":Date: $Date: %s $\n\n" % datetime.datetime.strftime(datetime.datetime.utcnow(), format="%F %X")

        # Add synopsis
        if self.synopsis:
            rst += "SYNOPSIS\n"
            rst += "========\n\n"
            for line in self.synopsis:
                rst += "%s\n\n" % line
            rst += "\n"

        # Add options
        rst += "OPTIONS\n"
        rst += "=======\n"

        for command in self.options:
            cmd  = ""
            if command["short_name"]:
                cmd += "-%s " % command["short_name"]
            cmd += "--%s" % command["long_name"]
            if command["arg_description"]:
                cmd += " %s" % command["arg_description"]

            rst += "%s\n" % cmd
            rst += "%s\n" % ("-" * len(cmd))
            rst += "%s\n" % command["description"]

            rst += "\n"

        return rst


def parse_arguments_from_c_file(filename):
    args = []

    try:
        content = open(filename, "r").read()
    except IOError:
        print "Error: Cannot open file %s" % filename
        return args

    re_cmd_entries = re.compile(r"\s*(static|const)[ ]+GOptionEntry[^{]*{(?P<entries>.*)\s*NULL\s*}[,]?\s*};", re.MULTILINE|re.DOTALL)
    match = re_cmd_entries.search(content)
    if not match:
        print "Warning: Cannot find GOptionEntry section in %s" % filename
        return args

    re_single_entry = re.compile(r"""{\s*"(?P<long_name>[^"]*)"\s*,        # long name
                                     \s*'?(?P<short_name>[^',]*)'?\s*,     # short name
                                     \s*[^,]*\s*,                          # flags
                                     \s*[^,]*\s*,                          # arg type
                                     \s*[^,]*\s*,                          # arg data pointer
                                     \s*("(?P<description>.*?)")\s*,       # description
                                     \s*"?(?P<arg_description>[^"}]*)"?    # arg description
                                  \s*},""", re.MULTILINE|re.DOTALL|re.VERBOSE)
    raw_entries_str = match.group("entries")

    start = 0
    entry_match = re_single_entry.search(raw_entries_str)
    i = 1
    while entry_match:
        long_name = entry_match.group("long_name").strip()
        short_name = entry_match.group("short_name").strip()
        description = entry_match.group("description").strip()
        arg_description = entry_match.group("arg_description").strip()

        # Normalize short name
        if short_name in ("NULL", "0", "\\0"):
            short_name = None

        # Normalize description
        description = description.replace('\\"', "\\\\'")
        description = description.replace("\\\\'", '"')
        description = description.replace('"', "")
        description = description.replace('\n', '')
        description = description.replace('\t', '')

        # Remove multiple whitespaces from description
        while True:
            new_description = description.replace("  ", " ")
            if new_description == description:
                break
            description = new_description
        description = description.strip()

        # Normalize arg_description
        if arg_description and (arg_description == "NULL" or arg_description == "0"):
            arg_description = None

        # Store option into list
        args.append({'long_name': long_name,
                     'short_name': short_name,
                     'description': description,
                     'arg_description': arg_description
                     })

        # Continue to next option
        i += 1
        start += entry_match.end(0)
        entry_match = re_single_entry.search(raw_entries_str[start:])
    # End while

    print >> sys.stderr, "Loaded %2d arguments" % (i,)
    return args



if __name__ == "__main__":
    parser = OptionParser('usage: %prog [options] <filename> [--mergerepo|--modifyrepo|--sqliterepo]')
    parser.add_option('-m', '--mergerepo', action="store_true", help="Gen rst for mergerepo")
    parser.add_option('-r', '--modifyrepo', action="store_true", help="Gen rst for modifyrepo")
    parser.add_option('-s', '--sqliterepo', action="store_true", help="Gen rst for sqliterepo")
    options, args = parser.parse_args()

    if len(args) < 1:
        print >> sys.stderr, "Error: Must specify a input filename. (Example: ../src/cmd_parser.c)"
        sys.exit(1)

    args = parse_arguments_from_c_file(args[0])

    if options.mergerepo:
        NAME = "mergerepo_c"
        info = Info(NAME,
                summary="Merge multiple rpm-md format repositories together",
                synopsis=["%s [options] --repo repo1 --repo repo2" % (NAME,)],
                options=args)
    elif options.modifyrepo:
        NAME = "modifyrepo_c"
        info = Info(NAME,
                summary="Modify a repomd.xml of rpm-md format repository",
                synopsis=["%s [options] <input metadata> <output repodata>" % (NAME,),
                          "%s --remove <metadata type> <output repodata>" % (NAME,),
                          "%s [options] --batchfile <batch file> <output repodata>" % (NAME,) ],
                options=args)
    elif options.sqliterepo:
        NAME = "sqliterepo_c"
        info = Info(NAME,
                summary="Generate sqlite db files for a repository in rpm-md format",
                synopsis=["%s [options] <repo_directory>" % (NAME,) ],
                options=args)
    else:
        NAME = "createrepo_c"
        info = Info(NAME,
                summary="Create rpm-md format (xml-rpm-metadata) repository",
                synopsis=["%s [options] <directory>" % (NAME,)],
                options=args)

    ret = info.gen_rst()
    if not ret:
        print >> sys.stderr, "Error: Rst has not been generated"
        sys.exit(1)

    print ret



from datetime import datetime
import unittest
import shutil
import tempfile
import os.path
import createrepo_c as cr

from .fixtures import *

class TestCaseUpdateInfo(unittest.TestCase):

    def test_updateinfo_setters(self):
        now = datetime.now()
        # Microseconds are always 0 in updateinfo
        now = datetime(now.year, now.month, now.day, now.hour, now.minute,
                       now.second, 0)

        ui = cr.UpdateInfo()
        self.assertTrue(ui)

        self.assertEqual(ui.updates, [])

        rec = cr.UpdateRecord()
        rec.fromstr = "from"
        rec.status = "status"
        rec.type = "type"
        rec.version = "version"
        rec.id = "id"
        rec.title = "title"
        rec.issued_date = now
        rec.updated_date = now
        rec.rights = "rights"
        rec.release = "release"
        rec.pushcount = "pushcount"
        rec.severity = "severity"
        rec.summary = "summary"
        rec.description = "description"
        rec.solution = "solution"

        ui.append(rec)

        self.assertEqual(len(ui.updates), 1)
        rec = ui.updates[0]

        self.assertEqual(rec.fromstr, "from")
        self.assertEqual(rec.status, "status")
        self.assertEqual(rec.type, "type")
        self.assertEqual(rec.version, "version")
        self.assertEqual(rec.id, "id")
        self.assertEqual(rec.title, "title")
        self.assertEqual(rec.issued_date, now)
        self.assertEqual(rec.updated_date, now)
        self.assertEqual(rec.rights, "rights")
        self.assertEqual(rec.release, "release")
        self.assertEqual(rec.pushcount, "pushcount")
        self.assertEqual(rec.severity, "severity")
        self.assertEqual(rec.summary, "summary")
        self.assertEqual(rec.description, "description")
        self.assertEqual(rec.solution, "solution")
        self.assertEqual(len(rec.references), 0)
        self.assertEqual(len(rec.collections), 0)

    def test_updateinfo_xml_dump_01(self):
        ui = cr.UpdateInfo()
        xml = ui.xml_dump()

        self.assertEqual(xml,
        """<?xml version="1.0" encoding="UTF-8"?>\n<updates/>\n""")

    def test_updateinfo_xml_dump_02(self):
        now = datetime.now()
        # Microseconds are always 0 in updateinfo
        now = datetime(now.year, now.month, now.day, now.hour, now.minute,
                       now.second, 0)

        ui = cr.UpdateInfo()
        xml = ui.xml_dump()

        rec = cr.UpdateRecord()
        rec.fromstr = "from"
        rec.status = "status"
        rec.type = "type"
        rec.version = "version"
        rec.id = "id"
        rec.title = "title"
        rec.issued_date = now
        rec.updated_date = now
        rec.rights = "rights"
        rec.release = "release"
        rec.pushcount = "pushcount"
        rec.severity = "severity"
        rec.summary = "summary"
        rec.description = "description"
        rec.solution = "solution"

        ui.append(rec)
        xml = ui.xml_dump()

        self.assertEqual(xml,
"""<?xml version="1.0" encoding="UTF-8"?>
<updates>
  <update from="from" status="status" type="type" version="version">
    <id>id</id>
    <title>title</title>
    <issued date="%(now)s"/>
    <updated date="%(now)s"/>
    <rights>rights</rights>
    <release>release</release>
    <pushcount>pushcount</pushcount>
    <severity>severity</severity>
    <summary>summary</summary>
    <description>description</description>
    <solution>solution</solution>
    <references/>
    <pkglist/>
  </update>
</updates>
""" % {"now": now.strftime("%Y-%m-%d %H:%M:%S")})

    def test_updateinfo_xml_dump_03(self):
        now = datetime.now()
        # Microseconds are always 0 in updateinfo
        now = datetime(now.year, now.month, now.day, now.hour, now.minute,
                       now.second, 0)

        mod = cr.UpdateCollectionModule()
        mod.name = "kangaroo"
        mod.stream = "0"
        mod.version = 18446744073709551615
        mod.context = "deadbeef"
        mod.arch = "x86"

        pkg = cr.UpdateCollectionPackage()
        pkg.name = "foo"
        pkg.version = "1.2"
        pkg.release = "3"
        pkg.epoch = "0"
        pkg.arch = "x86"
        pkg.src = "foo.src.rpm"
        pkg.filename = "foo.rpm"
        pkg.sum = "abcdef"
        pkg.sum_type = cr.SHA1
        pkg.reboot_suggested = True

        col = cr.UpdateCollection()
        col.shortname = "short name"
        col.name = "long name"
        col.module = mod
        col.append(pkg)

        ref = cr.UpdateReference()
        ref.href = "href"
        ref.id = "id"
        ref.type = "type"
        ref.title = "title"

        rec = cr.UpdateRecord()
        rec.fromstr = "from"
        rec.status = "status"
        rec.type = "type"
        rec.version = "version"
        rec.id = "id"
        rec.title = "title"
        rec.issued_date = now
        rec.updated_date = now
        rec.rights = "rights"
        rec.release = "release"
        rec.pushcount = "pushcount"
        rec.severity = "severity"
        rec.summary = "summary"
        rec.description = "description"
        rec.solution = "solution"
        rec.append_collection(col)
        rec.append_reference(ref)

        ui = cr.UpdateInfo()
        ui.append(rec)

        xml = ui.xml_dump()

        self.assertEqual(xml,
"""<?xml version="1.0" encoding="UTF-8"?>
<updates>
  <update from="from" status="status" type="type" version="version">
    <id>id</id>
    <title>title</title>
    <issued date="%(now)s"/>
    <updated date="%(now)s"/>
    <rights>rights</rights>
    <release>release</release>
    <pushcount>pushcount</pushcount>
    <severity>severity</severity>
    <summary>summary</summary>
    <description>description</description>
    <solution>solution</solution>
    <references>
      <reference href="href" id="id" type="type" title="title"/>
    </references>
    <pkglist>
      <collection short="short name">
        <name>long name</name>
        <module name="kangaroo" stream="0" version="18446744073709551615" context="deadbeef" arch="x86"/>
        <package name="foo" version="1.2" release="3" epoch="0" arch="x86" src="foo.src.rpm">
          <filename>foo.rpm</filename>
          <sum type="sha1">abcdef</sum>
          <reboot_suggested/>
        </package>
      </collection>
    </pkglist>
  </update>
</updates>
""" % {"now": now.strftime("%Y-%m-%d %H:%M:%S")})

    def test_updateinfo_xml_dump_04(self):
        now = datetime.now()
        # Microseconds are always 0 in updateinfo
        now = datetime(now.year, now.month, now.day, now.hour, now.minute,
                       now.second, 0)

        pkg = cr.UpdateCollectionPackage()
        pkg.name = "foo"
        pkg.version = "1.2"
        pkg.release = "3"
        pkg.epoch = "0"
        pkg.arch = "x86"
        pkg.src = "foo.src.rpm"
        pkg.filename = "foo.rpm"
        pkg.sum = "abcdef"
        pkg.sum_type = cr.SHA1
        pkg.reboot_suggested = True

        # Collection without module
        col = cr.UpdateCollection()
        col.shortname = "short name"
        col.name = "long name"
        col.append(pkg)

        ref = cr.UpdateReference()
        ref.href = "href"
        ref.id = "id"
        ref.type = "type"
        ref.title = "title"

        rec = cr.UpdateRecord()
        rec.fromstr = "from"
        rec.status = "status"
        rec.type = "type"
        rec.version = "version"
        rec.id = "id"
        rec.title = "title"
        rec.issued_date = now
        rec.updated_date = now
        rec.rights = "rights"
        rec.release = "release"
        rec.pushcount = "pushcount"
        rec.severity = "severity"
        rec.summary = "summary"
        rec.description = "description"
        rec.solution = "solution"
        rec.append_collection(col)
        rec.append_reference(ref)

        ui = cr.UpdateInfo()
        ui.append(rec)

        xml = ui.xml_dump()

        self.assertEqual(xml,
"""<?xml version="1.0" encoding="UTF-8"?>
<updates>
  <update from="from" status="status" type="type" version="version">
    <id>id</id>
    <title>title</title>
    <issued date="%(now)s"/>
    <updated date="%(now)s"/>
    <rights>rights</rights>
    <release>release</release>
    <pushcount>pushcount</pushcount>
    <severity>severity</severity>
    <summary>summary</summary>
    <description>description</description>
    <solution>solution</solution>
    <references>
      <reference href="href" id="id" type="type" title="title"/>
    </references>
    <pkglist>
      <collection short="short name">
        <name>long name</name>
        <package name="foo" version="1.2" release="3" epoch="0" arch="x86" src="foo.src.rpm">
          <filename>foo.rpm</filename>
          <sum type="sha1">abcdef</sum>
          <reboot_suggested/>
        </package>
      </collection>
    </pkglist>
  </update>
</updates>
""" % {"now": now.strftime("%Y-%m-%d %H:%M:%S")})

    def test_updateinfo_xml_dump_05(self):
        now = datetime.now()
        # Microseconds are always 0 in updateinfo
        now = datetime(now.year, now.month, now.day, now.hour, now.minute,
                       now.second, 0)

        # Collection module with unset fields
        mod = cr.UpdateCollectionModule()
        mod.version = 18446744073709551615
        mod.context = "deadbeef"
        mod.arch = "x86"

        pkg = cr.UpdateCollectionPackage()
        pkg.name = "foo"
        pkg.version = "1.2"
        pkg.release = "3"
        pkg.epoch = "0"
        pkg.arch = "x86"
        pkg.src = "foo.src.rpm"
        pkg.filename = "foo.rpm"
        pkg.sum = "abcdef"
        pkg.sum_type = cr.SHA1
        pkg.reboot_suggested = True

        col = cr.UpdateCollection()
        col.shortname = "short name"
        col.name = "long name"
        col.module = mod
        col.append(pkg)

        ref = cr.UpdateReference()
        ref.href = "href"
        ref.id = "id"
        ref.type = "type"
        ref.title = "title"

        rec = cr.UpdateRecord()
        rec.fromstr = "from"
        rec.status = "status"
        rec.type = "type"
        rec.version = "version"
        rec.id = "id"
        rec.title = "title"
        rec.issued_date = now
        rec.updated_date = now
        rec.rights = "rights"
        rec.release = "release"
        rec.pushcount = "pushcount"
        rec.severity = "severity"
        rec.summary = "summary"
        rec.description = "description"
        rec.solution = "solution"
        rec.append_collection(col)
        rec.append_reference(ref)

        ui = cr.UpdateInfo()
        ui.append(rec)

        xml = ui.xml_dump()

        self.assertEqual(xml,
"""<?xml version="1.0" encoding="UTF-8"?>
<updates>
  <update from="from" status="status" type="type" version="version">
    <id>id</id>
    <title>title</title>
    <issued date="%(now)s"/>
    <updated date="%(now)s"/>
    <rights>rights</rights>
    <release>release</release>
    <pushcount>pushcount</pushcount>
    <severity>severity</severity>
    <summary>summary</summary>
    <description>description</description>
    <solution>solution</solution>
    <references>
      <reference href="href" id="id" type="type" title="title"/>
    </references>
    <pkglist>
      <collection short="short name">
        <name>long name</name>
        <module version="18446744073709551615" context="deadbeef" arch="x86"/>
        <package name="foo" version="1.2" release="3" epoch="0" arch="x86" src="foo.src.rpm">
          <filename>foo.rpm</filename>
          <sum type="sha1">abcdef</sum>
          <reboot_suggested/>
        </package>
      </collection>
    </pkglist>
  </update>
</updates>
""" % {"now": now.strftime("%Y-%m-%d %H:%M:%S")})



#!/usr/bin/python

import re
import sys
import os.path
from optparse import OptionParser

VERSION_FILE_PATH = "VERSION.cmake"


def parse(root_dir):
    path = os.path.join(root_dir, VERSION_FILE_PATH)
    if not os.path.exists(path):
        print "File %s doesn't exists" % path
        return None

    content = open(path, "r").read()
    ver = {}
    ver['major'] = re.search('SET\s*\(CR_MAJOR\s+"(\d+)"', content).group(1)
    ver['minor'] = re.search('SET\s*\(CR_MINOR\s+"(\d+)"', content).group(1)
    ver['patch'] = re.search('SET\s*\(CR_PATCH\s+"(\d+)"', content).group(1)
    return ver


if __name__ == "__main__":
    parser = OptionParser("usage: %prog <project_root_dir> [--major|--minor|--patch]")
    parser.add_option("--major", action="store_true", help="Return major version")
    parser.add_option("--minor", action="store_true", help="Return minor version")
    parser.add_option("--patch", action="store_true", help="Return patch version")
    options, args = parser.parse_args()

    if len(args) != 1:
        parser.error("Must specify a project root directory")

    path = args[0]

    if not os.path.isdir(path):
        parser.error("Directory %s doesn't exists" % path)

    ver = parse(path)
    if ver is None:
        sys.exit(1)

    if options.major:
        print ver['major']
    elif options.minor:
        print ver['minor']
    elif options.patch:
        print ver['patch']
    else:
        print "%(major)s.%(minor)s.%(patch)s" % ver

    sys.exit(0)



import os
import os.path
import glob

from .fixtures import PACKAGES
from .base import BaseTestCase


class TestCaseCreaterepo_badparams(BaseTestCase):
    """Use case with bad commandline arguments"""

    def setup(self):
        self.indir_addpkg(PACKAGES[0])

    def test_01_createrepo_noinputdir(self):
        """No directory to index was specified"""
        res = self.run_cr("", c=True)
        self.assertTrue(res.rc)

    def test_02_createrepo_badinputdir(self):
        """Directory specified to index doesn't exist"""
        res = self.run_cr("somenonexistingdirectorytoindex/", c=True)
        self.assertTrue(res.rc)

    def test_03_createrepo_unknownparam(self):
        """Unknown param is specified"""
        res = self.run_cr(self.indir, "--someunknownparam", c=True)
        self.assertTrue(res.rc)

    def test_04_createrepo_badchecksumtype(self):
        """Unknown checksum type is specified"""
        res = self.run_cr(self.indir, "--checksum foobarunknownchecksum", c=True)
        self.assertTrue(res.rc)

    def test_05_createrepo_badcompressiontype(self):
        """Unknown compressin type is specified"""
        res = self.run_cr(self.indir, "--compress-type foobarunknowncompression", c=True)
        self.assertTrue(res.rc)

    def test_06_createrepo_badgroupfile(self):
        """Bad groupfile file specified"""
        res = self.run_cr(self.indir, "--groupfile badgroupfile", c=True)
        self.assertTrue(res.rc)

    def test_07_createrepo_badpkglist(self):
        """Bad pkglist file specified"""
        res = self.run_cr(self.indir, "--pkglist badpkglist", c=True)
        self.assertTrue(res.rc)

    def test_08_createrepo_retainoldmdbyagetogetherwithretainoldmd(self):
        """Both --retain-old-md-by-age and --retain-old-md are specified"""
        res = self.run_cr(self.indir, "--retain-old-md-by-age 1 --retain-old-md", c=True)
        self.assertTrue(res.rc)

    def test_09_createrepo_retainoldmdbyagewithbadage(self):
        """Both --retain-old-md-by-age and --retain-old-md are specified"""
        res = self.run_cr(self.indir, "--retain-old-md-by-age 55Z", c=True)
        self.assertTrue(res.rc)


class TestCaseCreaterepo_emptyrepo(BaseTestCase):
    """Empty input repository"""

    def setup(self):
        self.fn_comps = self.indir_mkfile("comps.xml", '<?xml version="1.0"?><comps/>')

    def test_01_createrepo(self):
        """Repo from empty directory"""
        res = self.assert_run_cr(self.indir, c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]{64}-primary.xml.gz$",
                                "[a-z0-9]{64}-filelists.xml.gz$",
                                "[a-z0-9]{64}-other.xml.gz$",
                                "[a-z0-9]{64}-primary.sqlite.bz2$",
                                "[a-z0-9]{64}-filelists.sqlite.bz2$",
                                "[a-z0-9]{64}-other.sqlite.bz2$",
                                ],
                               additional_files_allowed=False)

    def test_02_createrepo_database(self):
        """--database"""
        res = self.assert_run_cr(self.indir, "--database", c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]+-primary.xml.gz$",
                                "[a-z0-9]+-filelists.xml.gz$",
                                "[a-z0-9]+-other.xml.gz$",
                                "[a-z0-9]+-primary.sqlite.bz2$",
                                "[a-z0-9]+-filelists.sqlite.bz2$",
                                "[a-z0-9]+-other.sqlite.bz2$",
                                ],
                               additional_files_allowed=False)

    def test_03_createrepo_nodatabase(self):
        """--database"""
        res = self.assert_run_cr(self.indir, "--no-database", c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]+-primary.xml.gz$",
                                "[a-z0-9]+-filelists.xml.gz$",
                                "[a-z0-9]+-other.xml.gz$",
                                ],
                               additional_files_allowed=False)

    def test_04_createrepo_groupfile(self):
        """--groupfile"""
        res = self.assert_run_cr(self.indir, "--groupfile %s" % self.fn_comps, c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]+-primary.xml.gz$",
                                "[a-z0-9]+-filelists.xml.gz$",
                                "[a-z0-9]+-other.xml.gz$",
                                "[a-z0-9]+-primary.sqlite.bz2$",
                                "[a-z0-9]+-filelists.sqlite.bz2$",
                                "[a-z0-9]+-other.sqlite.bz2$",
                                "[a-z0-9]+-comps.xml$",
                                "[a-z0-9]+-comps.xml.gz$",
                                ],
                               additional_files_allowed=False)

    def test_05_createrepo_checksum(self):
        """--checksum sha and --groupfile"""
        res = self.assert_run_cr(self.indir,
                                 "--checksum %(checksum)s --groupfile %(groupfile)s" % {
                                    'checksum': "sha1",
                                    'groupfile': self.fn_comps },
                                 c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]{40}-primary.xml.gz$",
                                "[a-z0-9]{40}-filelists.xml.gz$",
                                "[a-z0-9]{40}-other.xml.gz$",
                                "[a-z0-9]{40}-primary.sqlite.bz2$",
                                "[a-z0-9]{40}-filelists.sqlite.bz2$",
                                "[a-z0-9]{40}-other.sqlite.bz2$",
                                "[a-z0-9]{40}-comps.xml$",
                                "[a-z0-9]{40}-comps.xml.gz$",
                                ],
                               additional_files_allowed=False)

    def test_06_createrepo_simplemdfilenames(self):
        """--simple-md-filenames and --groupfile"""
        res = self.assert_run_cr(self.indir,
                                 "--simple-md-filenames --groupfile %(groupfile)s" % {
                                    'groupfile': self.fn_comps },
                                 c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "primary.xml.gz$",
                                "filelists.xml.gz$",
                                "other.xml.gz$",
                                "primary.sqlite.bz2$",
                                "filelists.sqlite.bz2$",
                                "other.sqlite.bz2$",
                                "comps.xml$",
                                "comps.xml.gz$",
                                ],
                               additional_files_allowed=False)

    def test_07_createrepo_xz(self):
        """--xz and --groupfile"""
        res = self.assert_run_cr(self.indir,
                                 "--xz --groupfile %(groupfile)s" % {
                                    'groupfile': self.fn_comps },
                                 c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]{64}-primary.xml.gz$",
                                "[a-z0-9]{64}-filelists.xml.gz$",
                                "[a-z0-9]{64}-other.xml.gz$",
                                "[a-z0-9]{64}-primary.sqlite.xz$",
                                "[a-z0-9]{64}-filelists.sqlite.xz$",
                                "[a-z0-9]{64}-other.sqlite.xz$",
                                "[a-z0-9]{64}-comps.xml$",
                                "[a-z0-9]{64}-comps.xml.xz$",
                                ],
                               additional_files_allowed=False)

    def test_08_createrepo_compresstype_bz2(self):
        """--compress-type bz2"""
        res = self.assert_run_cr(self.indir, "--compress-type bz2", c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]{64}-primary.xml.gz$",
                                "[a-z0-9]{64}-filelists.xml.gz$",
                                "[a-z0-9]{64}-other.xml.gz$",
                                "[a-z0-9]{64}-primary.sqlite.bz2$",
                                "[a-z0-9]{64}-filelists.sqlite.bz2$",
                                "[a-z0-9]{64}-other.sqlite.bz2$",
                                ],
                               additional_files_allowed=False)

    def test_09_createrepo_compresstype_gz(self):
        """--compress-type bz2"""
        res = self.assert_run_cr(self.indir, "--compress-type gz", c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]{64}-primary.xml.gz$",
                                "[a-z0-9]{64}-filelists.xml.gz$",
                                "[a-z0-9]{64}-other.xml.gz$",
                                "[a-z0-9]{64}-primary.sqlite.gz$",
                                "[a-z0-9]{64}-filelists.sqlite.gz$",
                                "[a-z0-9]{64}-other.sqlite.gz$",
                                ],
                               additional_files_allowed=False)

    def test_10_createrepo_compresstype_xz(self):
        """--compress-type bz2"""
        res = self.assert_run_cr(self.indir, "--compress-type xz", c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]{64}-primary.xml.gz$",
                                "[a-z0-9]{64}-filelists.xml.gz$",
                                "[a-z0-9]{64}-other.xml.gz$",
                                "[a-z0-9]{64}-primary.sqlite.xz$",
                                "[a-z0-9]{64}-filelists.sqlite.xz$",
                                "[a-z0-9]{64}-other.sqlite.xz$",
                                ],
                               additional_files_allowed=False)

    def test_11_createrepo_repomd_checksum(self):
        """--checksum sha and --groupfile"""
        res = self.assert_run_cr(self.indir,
                                 "--repomd-checksum %(checksum)s --groupfile %(groupfile)s" % {
                                    'checksum': "sha1",
                                    'groupfile': self.fn_comps },
                                 c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]{40}-primary.xml.gz$",
                                "[a-z0-9]{40}-filelists.xml.gz$",
                                "[a-z0-9]{40}-other.xml.gz$",
                                "[a-z0-9]{40}-primary.sqlite.bz2$",
                                "[a-z0-9]{40}-filelists.sqlite.bz2$",
                                "[a-z0-9]{40}-other.sqlite.bz2$",
                                "[a-z0-9]{40}-comps.xml$",
                                "[a-z0-9]{40}-comps.xml.gz$",
                                ],
                               additional_files_allowed=False)

    def test_12_createrepo_repomd_checksum(self):
        """--checksum sha and --groupfile"""
        res = self.assert_run_cr(self.indir,
                                 "--checksum md5 --repomd-checksum %(checksum)s --groupfile %(groupfile)s" % {
                                    'checksum': "sha1",
                                    'groupfile': self.fn_comps },
                                 c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]{40}-primary.xml.gz$",
                                "[a-z0-9]{40}-filelists.xml.gz$",
                                "[a-z0-9]{40}-other.xml.gz$",
                                "[a-z0-9]{40}-primary.sqlite.bz2$",
                                "[a-z0-9]{40}-filelists.sqlite.bz2$",
                                "[a-z0-9]{40}-other.sqlite.bz2$",
                                "[a-z0-9]{40}-comps.xml$",
                                "[a-z0-9]{40}-comps.xml.gz$",
                                ],
                               additional_files_allowed=False)

    def test_13_createrepo_general_compress_type(self):
        """--checksum sha and --groupfile"""
        res = self.assert_run_cr(self.indir,
                                 "--general-compress-type %(compress_type)s --groupfile %(groupfile)s" % {
                                    'compress_type': "xz",
                                    'groupfile': self.fn_comps },
                                 c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]+-primary.xml.xz$",
                                "[a-z0-9]+-filelists.xml.xz$",
                                "[a-z0-9]+-other.xml.xz$",
                                "[a-z0-9]+-primary.sqlite.xz$",
                                "[a-z0-9]+-filelists.sqlite.xz$",
                                "[a-z0-9]+-other.sqlite.xz$",
                                "[a-z0-9]+-comps.xml$",
                                "[a-z0-9]+-comps.xml.xz$",
                                ],
                               additional_files_allowed=False)

class TestCaseCreaterepo_packagedirstructure(BaseTestCase):
    """Use case with various directory structure of packages"""
    def test_01_createrepo_check_directories_recursively(self):
        self.indir_makedirs("a")
        self.indir_makedirs("a/b")
        self.indir_makedirs("a/b/c")
        self.copy_pkg(PACKAGES[0], self.indir + "/a")
        self.copy_pkg(PACKAGES[1], self.indir + "/a/b")
        self.copy_pkg(PACKAGES[2], self.indir + "/a/b/c")

        res = self.assert_run_cr(self.indir, "", c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]+-primary.xml.gz$",
                                "[a-z0-9]+-filelists.xml.gz$",
                                "[a-z0-9]+-other.xml.gz$",
                                "[a-z0-9]+-primary.sqlite.bz2$",
                                "[a-z0-9]+-filelists.sqlite.bz2$",
                                "[a-z0-9]+-other.sqlite.bz2$",
                                ],
                               additional_files_allowed=False)

        primary_path = glob.glob(os.path.join(res.outdir, "repodata/*-primary.xml.gz"))[0]
        self.assert_file_contains_line(primary_path, "<name>Archer</name>")
        self.assert_file_contains_line(primary_path, "<name>super_kernel</name>")
        self.assert_file_contains_line(primary_path, "<name>fake_bash</name>")
        
    def test_02_createrepo_check_directories_recursively_with_rpm_suffix(self):
        self.indir_makedirs("a")
        self.indir_makedirs('a/b.rpm')
        self.indir_makedirs("a/b.rpm/c")
        self.copy_pkg(PACKAGES[0], self.indir + "/a")
        self.copy_pkg(PACKAGES[1], self.indir + "/a/b.rpm")
        self.copy_pkg(PACKAGES[2], self.indir + "/a/b.rpm/c")

        res = self.assert_run_cr(self.indir, "", c=True)
        self.assert_repo_sanity(res.outdir)
        self.assert_repo_files(res.outdir,
                               ["repomd.xml$",
                                "[a-z0-9]+-primary.xml.gz$",
                                "[a-z0-9]+-filelists.xml.gz$",
                                "[a-z0-9]+-other.xml.gz$",
                                "[a-z0-9]+-primary.sqlite.bz2$",
                                "[a-z0-9]+-filelists.sqlite.bz2$",
                                "[a-z0-9]+-other.sqlite.bz2$",
                                ],
                               additional_files_allowed=False)

        primary_path = glob.glob(os.path.join(res.outdir, "repodata/*-primary.xml.gz"))[0]
        self.assert_file_contains_line(primary_path, "<name>Archer</name>")
        self.assert_file_contains_line(primary_path, "<name>super_kernel</name>")
        self.assert_file_contains_line(primary_path, "<name>fake_bash</name>")



from distutils.core import setup
import sys

# This is a simple and fragile way of passing the current version
# from cmake to setup as I assume no one else will use this.
#
# This script has to have the version always specified as last argument.
version = sys.argv.pop()

setup(
    name='createrepo_c',
    description='C implementation of createrepo',
    version=version,
    license='GPLv2+',
    author='RPM Software Management',
    author_email='rpm-ecosystem@lists.rpm.org',
    url='https://github.com/rpm-software-management',
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: C',
        'Topic :: System :: Software Distribution',
        'Topic :: System :: Systems Administration',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)



#!/usr/bin/env python

import os
import os.path
import optparse

import createrepo_c as cr


def parse_updateinfo(path):
    uinfo = cr.UpdateInfo(path)
    for update in uinfo.updates:
        print "From:         %s" % update.fromstr
        print "Status:       %s" % update.status
        print "Type:         %s" % update.type
        print "Version:      %s" % update.version
        print "Id:           %s" % update.id
        print "Title:        %s" % update.title
        print "Issued date:  %s" % update.issued_date
        print "Updated date: %s" % update.updated_date
        print "Rights:       %s" % update.rights
        print "Release:      %s" % update.release
        print "Pushcount:    %s" % update.pushcount
        print "Severity:     %s" % update.severity
        print "Summary:      %s" % update.summary
        print "Description:  %s" % update.description
        print "Solution:     %s" % update.solution
        print "References:"
        for ref in update.references:
            print "  Href:  %s" % ref.href
            print "  Id:    %s" % ref.id
            print "  Type:  %s" % ref.type
            print "  Title: %s" % ref.title
            print "  ----------------------------"
        print "Pkglist (collections):"
        for col in update.collections:
            print "  Short: %s" % col.shortname
            print "  name:  %s" % col.name
            print "  Packages:"
            for pkg in col.packages:
                print "    Name:     %s" % pkg.name
                print "    Version:  %s" % pkg.version
                print "    Release:  %s" % pkg.release
                print "    Epoch:    %s" % pkg.epoch
                print "    Arch:     %s" % pkg.arch
                print "    Src:      %s" % pkg.src
                print "    Filename: %s" % pkg.filename
                print "    Sum:      %s" % pkg.sum
                print "    Sum type: %s (%s)" % (pkg.sum_type, cr.checksum_name_str(pkg.sum_type))
                print "    Reboot suggested: %s" % pkg.reboot_suggested
            print "  ----------------------------"
        print "=============================="


if __name__ == "__main__":
    parser = optparse.OptionParser(usage="%prog PATH_TO_UPDATEINFO")
    options, args = parser.parse_args()
    if len(args) != 1:
        parser.error("You have to specify exactly one update info")
    parse_updateinfo(args[0])



import unittest
import shutil
import tempfile
import os.path
import createrepo_c as cr

from .fixtures import *

class TestCaseXmlFile(unittest.TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp(prefix="createrepo_ctest-")

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def test_xmlfile_basic_operations(self):
        pri = cr.XmlFile(self.tmpdir+"/primary.xml.gz",
                         cr.XMLFILE_PRIMARY,
                         cr.GZ_COMPRESSION,
                         None)
        self.assertTrue(pri)
        self.assertTrue(os.path.isfile(self.tmpdir+"/primary.xml.gz"))

        fil = cr.XmlFile(self.tmpdir+"/filelists.xml.gz",
                         cr.XMLFILE_FILELISTS,
                         cr.GZ_COMPRESSION,
                         None)
        self.assertTrue(fil)
        self.assertTrue(os.path.isfile(self.tmpdir+"/filelists.xml.gz"))

        oth = cr.XmlFile(self.tmpdir+"/other.xml.gz",
                         cr.XMLFILE_OTHER,
                         cr.GZ_COMPRESSION,
                         None)
        self.assertTrue(oth)
        self.assertTrue(os.path.isfile(self.tmpdir+"/other.xml.gz"))

    def test_xmlfile_operations_on_closed_file(self):
        # Already closed file
        path = os.path.join(self.tmpdir, "primary.xml.gz")
        pkg = cr.package_from_rpm(PKG_ARCHER_PATH)
        self.assertTrue(pkg)

        f = cr.PrimaryXmlFile(path, cr.GZ_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.close()
        self.assertRaises(cr.CreaterepoCError, f.set_num_of_pkgs, 1)
        self.assertRaises(cr.CreaterepoCError, f.add_pkg, pkg)
        self.assertRaises(cr.CreaterepoCError, f.add_chunk, "<chunk>text</chunk>")
        f.close() # No error should be raised
        del(f)    # No error should be raised

    def test_xmlfile_error_cases(self):
        path = os.path.join(self.tmpdir, "foofile")
        self.assertFalse(os.path.exists(path))

        # Bad file type
        self.assertRaises(ValueError, cr.XmlFile, path, 86,
                          cr.GZ_COMPRESSION, None)
        self.assertFalse(os.path.exists(path))

        # Bad compression type
        self.assertRaises(ValueError, cr.XmlFile, path,
                          cr.XMLFILE_PRIMARY, 678, None)
        self.assertFalse(os.path.exists(path))

        # Bad contentstat object
        self.assertRaises(TypeError, cr.XmlFile, path,
                          cr.XMLFILE_PRIMARY, cr.GZ_COMPRESSION, "foo")
        self.assertFalse(os.path.exists(path))

        # Non existing path
        self.assertRaises(cr.CreaterepoCError, cr.PrimaryXmlFile,
                          "foobar/foo/xxx/cvydmaticxuiowe")

        # Already existing file
        open(path, "w").write("foobar")
        self.assertRaises(IOError, cr.PrimaryXmlFile, path)

    def test_xmlfile_no_compression(self):
        path = os.path.join(self.tmpdir, "primary.xml")
        f = cr.PrimaryXmlFile(path, cr.NO_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.close()

        content = open(path).read()
        self.assertEqual(content,
"""<?xml version="1.0" encoding="UTF-8"?>
<metadata xmlns="http://linux.duke.edu/metadata/common" xmlns:rpm="http://linux.duke.edu/metadata/rpm" packages="0">
</metadata>""")

    def test_xmlfile_gz_compression(self):
        path = os.path.join(self.tmpdir, "primary.xml.gz")
        f = cr.PrimaryXmlFile(path, cr.GZ_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.close()

        import gzip
        content = gzip.open(path).read().decode('utf-8')
        self.assertEqual(content,
"""<?xml version="1.0" encoding="UTF-8"?>
<metadata xmlns="http://linux.duke.edu/metadata/common" xmlns:rpm="http://linux.duke.edu/metadata/rpm" packages="0">
</metadata>""")

    def test_xmlfile_bz2_compression(self):
        path = os.path.join(self.tmpdir, "primary.xml.bz2")
        f = cr.PrimaryXmlFile(path, cr.BZ2_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.close()

        import bz2
        content = bz2.decompress(open(path, 'rb').read()).decode('utf-8')
        self.assertEqual(content,
"""<?xml version="1.0" encoding="UTF-8"?>
<metadata xmlns="http://linux.duke.edu/metadata/common" xmlns:rpm="http://linux.duke.edu/metadata/rpm" packages="0">
</metadata>""")

    def test_xmlfile_xz_compression(self):
        path = os.path.join(self.tmpdir, "primary.xml.xz")
        f = cr.PrimaryXmlFile(path, cr.XZ_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.close()

        import subprocess
        p = subprocess.Popen(["unxz", "--stdout", path], stdout=subprocess.PIPE)
        content = p.stdout.read().decode('utf-8')
        self.assertEqual(content,
"""<?xml version="1.0" encoding="UTF-8"?>
<metadata xmlns="http://linux.duke.edu/metadata/common" xmlns:rpm="http://linux.duke.edu/metadata/rpm" packages="0">
</metadata>""")

    def test_xmlfile_zck_compression(self):
        if cr.HAS_ZCK == 0:
            return

        path = os.path.join(self.tmpdir, "primary.xml.zck")
        f = cr.PrimaryXmlFile(path, cr.ZCK_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.close()

        import subprocess
        p = subprocess.Popen(["unzck", "--stdout", path], stdout=subprocess.PIPE)
        content = p.stdout.read().decode('utf-8')
        self.assertEqual(content,
"""<?xml version="1.0" encoding="UTF-8"?>
<metadata xmlns="http://linux.duke.edu/metadata/common" xmlns:rpm="http://linux.duke.edu/metadata/rpm" packages="0">
</metadata>""")

    def test_xmlfile_set_num_of_pkgs(self):
        path = os.path.join(self.tmpdir, "primary.xml")
        f = cr.PrimaryXmlFile(path, cr.NO_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.set_num_of_pkgs(22)
        f.close()

        content = open(path).read()
        self.assertEqual(content,
"""<?xml version="1.0" encoding="UTF-8"?>
<metadata xmlns="http://linux.duke.edu/metadata/common" xmlns:rpm="http://linux.duke.edu/metadata/rpm" packages="22">
</metadata>""")

    def test_xmlfile_add_pkg(self):
        pkg = cr.package_from_rpm(PKG_ARCHER_PATH)
        self.assertTrue(pkg)
        pkg.time_file = 111

        # Primary
        path = os.path.join(self.tmpdir, "primary.xml")
        f = cr.PrimaryXmlFile(path, cr.NO_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.add_pkg(pkg)
        self.assertRaises(TypeError, f.add_pkg, None)
        self.assertRaises(TypeError, f.add_pkg, 123)
        self.assertRaises(TypeError, f.add_pkg, "foo")
        self.assertRaises(TypeError, f.add_pkg, [456])
        f.close()

        self.assertTrue(os.path.isfile(path))
        self.assertEqual(open(path).read(),
"""<?xml version="1.0" encoding="UTF-8"?>
<metadata xmlns="http://linux.duke.edu/metadata/common" xmlns:rpm="http://linux.duke.edu/metadata/rpm" packages="0">
<package type="rpm">
  <name>Archer</name>
  <arch>x86_64</arch>
  <version epoch="2" ver="3.4.5" rel="6"/>
  <checksum type="sha256" pkgid="YES">4e0b775220c67f0f2c1fd2177e626b9c863a098130224ff09778ede25cea9a9e</checksum>
  <summary>Complex package.</summary>
  <description>Archer package</description>
  <packager>Sterling Archer</packager>
  <url>http://soo_complex_package.eu/</url>
  <time file="111" build="1365416480"/>
  <size package="3101" installed="0" archive="544"/>
  <location href=""/>
  <format>
    <rpm:license>GPL</rpm:license>
    <rpm:vendor>ISIS</rpm:vendor>
    <rpm:group>Development/Tools</rpm:group>
    <rpm:buildhost>localhost.localdomain</rpm:buildhost>
    <rpm:sourcerpm>Archer-3.4.5-6.src.rpm</rpm:sourcerpm>
    <rpm:header-range start="280" end="2865"/>
    <rpm:provides>
      <rpm:entry name="bara" flags="LE" epoch="0" ver="22"/>
      <rpm:entry name="barb" flags="GE" epoch="0" ver="11.22.33" rel="44"/>
      <rpm:entry name="barc" flags="EQ" epoch="0" ver="33"/>
      <rpm:entry name="bard" flags="LT" epoch="0" ver="44"/>
      <rpm:entry name="bare" flags="GT" epoch="0" ver="55"/>
      <rpm:entry name="Archer" flags="EQ" epoch="2" ver="3.4.5" rel="6"/>
      <rpm:entry name="Archer(x86-64)" flags="EQ" epoch="2" ver="3.4.5" rel="6"/>
    </rpm:provides>
    <rpm:requires>
      <rpm:entry name="fooa" flags="LE" epoch="0" ver="2"/>
      <rpm:entry name="foob" flags="GE" epoch="0" ver="1.0.0" rel="1"/>
      <rpm:entry name="fooc" flags="EQ" epoch="0" ver="3"/>
      <rpm:entry name="food" flags="LT" epoch="0" ver="4"/>
      <rpm:entry name="fooe" flags="GT" epoch="0" ver="5"/>
      <rpm:entry name="foof" flags="EQ" epoch="0" ver="6" pre="1"/>
    </rpm:requires>
    <rpm:conflicts>
      <rpm:entry name="bba" flags="LE" epoch="0" ver="2222"/>
      <rpm:entry name="bbb" flags="GE" epoch="0" ver="1111.2222.3333" rel="4444"/>
      <rpm:entry name="bbc" flags="EQ" epoch="0" ver="3333"/>
      <rpm:entry name="bbd" flags="LT" epoch="0" ver="4444"/>
      <rpm:entry name="bbe" flags="GT" epoch="0" ver="5555"/>
    </rpm:conflicts>
    <rpm:obsoletes>
      <rpm:entry name="aaa" flags="LE" epoch="0" ver="222"/>
      <rpm:entry name="aab" flags="GE" epoch="0" ver="111.2.3" rel="4"/>
      <rpm:entry name="aac" flags="EQ" epoch="0" ver="333"/>
      <rpm:entry name="aad" flags="LT" epoch="0" ver="444"/>
      <rpm:entry name="aae" flags="GT" epoch="0" ver="555"/>
    </rpm:obsoletes>
    <file>/usr/bin/complex_a</file>
  </format>
</package>
</metadata>""")

        # Filelists
        path = os.path.join(self.tmpdir, "filelists.xml")
        f = cr.FilelistsXmlFile(path, cr.NO_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.add_pkg(pkg)
        self.assertRaises(TypeError, f.add_pkg, None)
        self.assertRaises(TypeError, f.add_pkg, 123)
        self.assertRaises(TypeError, f.add_pkg, "foo")
        self.assertRaises(TypeError, f.add_pkg, [456])
        f.close()

        self.assertTrue(os.path.isfile(path))
        self.assertEqual(open(path).read(),
"""<?xml version="1.0" encoding="UTF-8"?>
<filelists xmlns="http://linux.duke.edu/metadata/filelists" packages="0">
<package pkgid="4e0b775220c67f0f2c1fd2177e626b9c863a098130224ff09778ede25cea9a9e" name="Archer" arch="x86_64">
  <version epoch="2" ver="3.4.5" rel="6"/>
  <file>/usr/bin/complex_a</file>
  <file type="dir">/usr/share/doc/Archer-3.4.5</file>
  <file>/usr/share/doc/Archer-3.4.5/README</file>
</package>
</filelists>""")

        # Other
        path = os.path.join(self.tmpdir, "other.xml")
        f = cr.OtherXmlFile(path, cr.NO_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.add_pkg(pkg)
        self.assertRaises(TypeError, f.add_pkg, None)
        self.assertRaises(TypeError, f.add_pkg, 123)
        self.assertRaises(TypeError, f.add_pkg, "foo")
        self.assertRaises(TypeError, f.add_pkg, [456])
        f.close()

        self.assertTrue(os.path.isfile(path))
        self.assertEqual(open(path).read(),
"""<?xml version="1.0" encoding="UTF-8"?>
<otherdata xmlns="http://linux.duke.edu/metadata/other" packages="0">
<package pkgid="4e0b775220c67f0f2c1fd2177e626b9c863a098130224ff09778ede25cea9a9e" name="Archer" arch="x86_64">
  <version epoch="2" ver="3.4.5" rel="6"/>
  <changelog author="Tomas Mlcoch &lt;tmlcoch@redhat.com&gt; - 1.1.1-1" date="1334664000">- First changelog.</changelog>
  <changelog author="Tomas Mlcoch &lt;tmlcoch@redhat.com&gt; - 2.2.2-2" date="1334750400">- That was totally ninja!</changelog>
  <changelog author="Tomas Mlcoch &lt;tmlcoch@redhat.com&gt; - 3.3.3-3" date="1365422400">- 3. changelog.</changelog>
</package>
</otherdata>""")

    def test_xmlfile_add_chunk(self):
        chunk = "  <chunk>Some XML chunk</chunk>\n"

        # Primary
        path = os.path.join(self.tmpdir, "primary.xml")
        f = cr.PrimaryXmlFile(path, cr.NO_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.add_chunk(chunk)
        self.assertRaises(TypeError, f.add_chunk, None)
        self.assertRaises(TypeError, f.add_chunk, 123)
        self.assertRaises(TypeError, f.add_chunk, [1])
        self.assertRaises(TypeError, f.add_chunk, ["foo"])
        f.close()

        self.assertTrue(os.path.isfile(path))
        self.assertEqual(open(path).read(),
"""<?xml version="1.0" encoding="UTF-8"?>
<metadata xmlns="http://linux.duke.edu/metadata/common" xmlns:rpm="http://linux.duke.edu/metadata/rpm" packages="0">
  <chunk>Some XML chunk</chunk>
</metadata>""")

        # Filelists
        path = os.path.join(self.tmpdir, "filelists.xml")
        f = cr.FilelistsXmlFile(path, cr.NO_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.add_chunk(chunk)
        self.assertRaises(TypeError, f.add_chunk, None)
        self.assertRaises(TypeError, f.add_chunk, 123)
        self.assertRaises(TypeError, f.add_chunk, [1])
        self.assertRaises(TypeError, f.add_chunk, ["foo"])
        f.close()

        self.assertTrue(os.path.isfile(path))
        self.assertEqual(open(path).read(),
"""<?xml version="1.0" encoding="UTF-8"?>
<filelists xmlns="http://linux.duke.edu/metadata/filelists" packages="0">
  <chunk>Some XML chunk</chunk>
</filelists>""")

        # Other
        path = os.path.join(self.tmpdir, "other.xml")
        f = cr.OtherXmlFile(path, cr.NO_COMPRESSION)
        self.assertTrue(f)
        self.assertTrue(os.path.isfile(path))
        f.add_chunk(chunk)
        self.assertRaises(TypeError, f.add_chunk, None)
        self.assertRaises(TypeError, f.add_chunk, 123)
        self.assertRaises(TypeError, f.add_chunk, [1])
        self.assertRaises(TypeError, f.add_chunk, ["foo"])
        f.close()

        self.assertTrue(os.path.isfile(path))
        self.assertEqual(open(path).read(),
"""<?xml version="1.0" encoding="UTF-8"?>
<otherdata xmlns="http://linux.duke.edu/metadata/other" packages="0">
  <chunk>Some XML chunk</chunk>
</otherdata>""")



from datetime import datetime
import unittest
import shutil
import tempfile
import os.path
import createrepo_c as cr

from .fixtures import *

class TestCaseUpdateRecord(unittest.TestCase):

    def test_updaterecord_setters(self):
        now = datetime.now()
        # Microseconds are always 0 in updateinfo
        now = datetime(now.year, now.month, now.day, now.hour, now.minute,
                       now.second, 0)

        rec = cr.UpdateRecord()
        self.assertTrue(rec)

        self.assertEqual(rec.fromstr, None)
        self.assertEqual(rec.status, None)
        self.assertEqual(rec.type, None)
        self.assertEqual(rec.version, None)
        self.assertEqual(rec.id, None)
        self.assertEqual(rec.title, None)
        self.assertEqual(rec.issued_date, None)
        self.assertEqual(rec.updated_date, None)
        self.assertEqual(rec.rights, None)
        self.assertEqual(rec.release, None)
        self.assertEqual(rec.pushcount, None)
        self.assertEqual(rec.severity, None)
        self.assertEqual(rec.summary, None)
        self.assertEqual(rec.description, None)
        self.assertEqual(rec.solution, None)
        self.assertEqual(rec.references, [])
        self.assertEqual(rec.collections, [])

        ref = cr.UpdateReference()
        ref.href = "href"
        ref.id = "id"
        ref.type = "type"
        ref.title = "title"

        col = cr.UpdateCollection()
        col.shortname = "short name"
        col.name = "long name"

        rec.fromstr = "from"
        rec.status = "status"
        rec.type = "type"
        rec.version = "version"
        rec.id = "id"
        rec.title = "title"
        rec.issued_date = now
        rec.updated_date = now
        rec.rights = "rights"
        rec.release = "release"
        rec.pushcount = "pushcount"
        rec.severity = "severity"
        rec.summary = "summary"
        rec.description = "description"
        rec.solution = "solution"
        rec.append_reference(ref)
        rec.append_collection(col)

        self.assertEqual(rec.fromstr, "from")
        self.assertEqual(rec.status, "status")
        self.assertEqual(rec.type, "type")
        self.assertEqual(rec.version, "version")
        self.assertEqual(rec.id, "id")
        self.assertEqual(rec.title, "title")
        self.assertEqual(rec.issued_date, now)
        self.assertEqual(rec.updated_date, now)
        self.assertEqual(rec.rights, "rights")
        self.assertEqual(rec.release, "release")
        self.assertEqual(rec.pushcount, "pushcount")
        self.assertEqual(rec.severity, "severity")
        self.assertEqual(rec.summary, "summary")
        self.assertEqual(rec.description, "description")
        self.assertEqual(rec.solution, "solution")
        self.assertEqual(len(rec.references), 1)
        self.assertEqual(len(rec.collections), 1)

        ref = rec.references[0]
        self.assertEqual(ref.href, "href")
        self.assertEqual(ref.id, "id")
        self.assertEqual(ref.type, "type")
        self.assertEqual(ref.title, "title")

        col = rec.collections[0]
        self.assertEqual(col.shortname, "short name")
        self.assertEqual(col.name, "long name")
        self.assertEqual(len(col.packages), 0)

    def test_xml_dump_updaterecord(self):
        now = datetime.now()
        # Microseconds are always 0 in updateinfo
        now = datetime(now.year, now.month, now.day, now.hour, now.minute,
                       now.second, 0)

        rec = cr.UpdateRecord()
        rec.fromstr = "from"
        rec.status = "status"
        rec.type = "type"
        rec.version = "version"
        rec.id = "id"
        rec.title = "title"
        rec.issued_date = now
        rec.updated_date = now
        rec.rights = "rights"
        rec.release = "release"
        rec.pushcount = "pushcount"
        rec.severity = "severity"
        rec.summary = "summary"
        rec.description = "description"
        rec.solution = "solution"

        xml = cr.xml_dump_updaterecord(rec)
        self.assertEqual(xml,
"""  <update from="from" status="status" type="type" version="version">
    <id>id</id>
    <title>title</title>
    <issued date="%(now)s"/>
    <updated date="%(now)s"/>
    <rights>rights</rights>
    <release>release</release>
    <pushcount>pushcount</pushcount>
    <severity>severity</severity>
    <summary>summary</summary>
    <description>description</description>
    <solution>solution</solution>
    <references/>
    <pkglist/>
  </update>
""" % {"now": now.strftime("%Y-%m-%d %H:%M:%S")})



import re
import unittest
import shutil
import tempfile
import os.path
import createrepo_c as cr

from .fixtures import *

class TestCaseXmlParserPrimary(unittest.TestCase):

    def test_xml_parser_primary_repo01(self):

        userdata = {
                "pkgs": [],
                "pkgcb_calls": 0,
                "warnings": []
            }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        def pkgcb(pkg):
            userdata["pkgcb_calls"] += 1

        def warningcb(warn_type, msg):
            userdata["warnings"].append((warn_type, msg))

        cr.xml_parse_primary(REPO_01_PRIXML, newpkgcb, pkgcb, warningcb, 1)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]],
            ['super_kernel'])
        self.assertEqual(userdata["pkgcb_calls"], 1)
        self.assertEqual(userdata["warnings"], [])

        pkg = userdata["pkgs"][0]
        self.assertEqual(pkg.pkgId, "152824bff2aa6d54f429d43e87a3ff3a0286505c6d93ec87692b5e3a9e3b97bf")
        self.assertEqual(pkg.name, "super_kernel")
        self.assertEqual(pkg.arch, "x86_64")
        self.assertEqual(pkg.version, "6.0.1")
        self.assertEqual(pkg.epoch, "0")
        self.assertEqual(pkg.release, "2")
        self.assertEqual(pkg.summary, "Test package")
        self.assertEqual(pkg.description, "This package has provides, requires, obsoletes, conflicts options.")
        self.assertEqual(pkg.url, "http://so_super_kernel.com/it_is_awesome/yep_it_really_is")
        self.assertEqual(pkg.time_file, 1334667003)
        self.assertEqual(pkg.time_build, 1334667003)
        self.assertEqual(pkg.rpm_license, "LGPLv2")
        self.assertEqual(pkg.rpm_vendor, None)
        self.assertEqual(pkg.rpm_group, "Applications/System")
        self.assertEqual(pkg.rpm_buildhost, "localhost.localdomain")
        self.assertEqual(pkg.rpm_sourcerpm, "super_kernel-6.0.1-2.src.rpm")
        self.assertEqual(pkg.rpm_header_start, 280)
        self.assertEqual(pkg.rpm_header_end, 2637)
        self.assertEqual(pkg.rpm_packager, None)
        self.assertEqual(pkg.size_package, 2845)
        self.assertEqual(pkg.size_installed, 0)
        self.assertEqual(pkg.size_archive, 404)
        self.assertEqual(pkg.location_href, "super_kernel-6.0.1-2.x86_64.rpm")
        self.assertEqual(pkg.location_base, None)
        self.assertEqual(pkg.checksum_type, "sha256")
        self.assertEqual(pkg.requires,
                [('bzip2', 'GE', '0', '1.0.0', None, True),
                 ('expat', None, None, None, None, True),
                 ('glib', 'GE', '0', '2.26.0', None, False),
                 ('zlib', None, None, None, None, False)])
        self.assertEqual(pkg.provides,
                [('not_so_super_kernel', 'LT', '0', '5.8.0', None, False),
                 ('super_kernel', 'EQ', '0', '6.0.0', None, False),
                 ('super_kernel', 'EQ', '0', '6.0.1', '2', False),
                 ('super_kernel(x86-64)', 'EQ', '0', '6.0.1', '2', False)])
        self.assertEqual(pkg.conflicts,
                [('kernel', None, None, None, None, False),
                 ('super_kernel', 'EQ', '0', '5.0.0', None, False),
                 ('super_kernel', 'LT', '0', '4.0.0', None, False)])
        self.assertEqual(pkg.obsoletes,
                [('kernel', None, None, None, None, False),
                 ('super_kernel', 'EQ', '0', '5.9.0', None, False)])
        self.assertEqual(pkg.files,
                [(None, '/usr/bin/', 'super_kernel')])
        self.assertEqual(pkg.changelogs, [])


    def test_xml_parser_primary_repo02(self):

        userdata = {
                "pkgs": [],
                "pkgcb_calls": 0,
                "warnings": []
            }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        def pkgcb(pkg):
            userdata["pkgcb_calls"] += 1

        def warningcb(warn_type, msg):
            userdata["warnings"].append((warn_type, msg))

        cr.xml_parse_primary(REPO_02_PRIXML, newpkgcb, pkgcb, warningcb, 1)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]],
            ['fake_bash', 'super_kernel'])
        self.assertEqual(userdata["pkgcb_calls"], 2)
        self.assertEqual(userdata["warnings"], [])

    def test_xml_parser_primary_repo02_only_pkgcb(self):

        pkgs = []

        def pkgcb(pkg):
            pkgs.append(pkg)

        cr.xml_parse_primary(REPO_02_PRIXML, None, pkgcb, None, 1)

        self.assertEqual([pkg.name for pkg in pkgs],
            ['fake_bash', 'super_kernel'])

    def test_xml_parser_primary_repo02_no_cbs(self):
        self.assertRaises(ValueError,
                          cr.xml_parse_primary,
                          REPO_02_PRIXML, None, None, None, 1)

    def test_xml_parser_primary_warnings(self):

        userdata = {
                "pkgs": [],
                "warnings": []
            }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        def warningcb(warn_type, msg):
            userdata["warnings"].append((warn_type, msg))

        cr.xml_parse_primary(PRIMARY_MULTI_WARN_00_PATH,
                             newpkgcb,
                             None,
                             warningcb,
                             1)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]],
            ['fake_bash', 'super_kernel'])
        self.assertEqual(userdata["warnings"],
            [(0, 'Unknown element "fooelement"'),
             (1, 'Missing attribute "type" of a package element'),
             (0, 'Unknown element "foo"'),
             (3, 'Conversion of "foobar" to integer failed'),
             (2, 'Unknown file type "xxx"'),
             (0, 'Unknown element "bar"')])

    def test_xml_parser_primary_error(self):

        userdata = { "pkgs": [] }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_primary,
                          PRIMARY_ERROR_00_PATH, newpkgcb, None, None, 1)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]], ['fake_bash'])

    def test_xml_parser_primary_newpkgcb_abort(self):
        def newpkgcb(pkgId, name, arch):
            raise Error("Foo error")
        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_primary,
                          REPO_02_PRIXML, newpkgcb, None, None, 1)

    def test_xml_parser_primary_pkgcb_abort(self):
        def newpkgcb(pkgId, name, arch):
            return cr.Package()
        def pkgcb(pkg):
            raise Error("Foo error")
        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_primary,
                          REPO_02_PRIXML, newpkgcb, pkgcb, None, 1)

    def test_xml_parser_primary_warningcb_abort(self):
        def newpkgcb(pkgId, name, arch):
            return cr.Package()
        def warningcb(type, msg):
            raise Error("Foo error")
        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_primary,
                          PRIMARY_MULTI_WARN_00_PATH,
                          newpkgcb, None, warningcb, 1)

class TestCaseXmlParserFilelists(unittest.TestCase):

    def test_xml_parser_filelists_repo01(self):

        userdata = {
                "pkgs": [],
                "pkgcb_calls": 0,
                "warnings": []
            }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        def pkgcb(pkg):
            userdata["pkgcb_calls"] += 1

        def warningcb(warn_type, msg):
            userdata["warnings"].append((warn_type, msg))

        cr.xml_parse_filelists(REPO_01_FILXML, newpkgcb, pkgcb, warningcb)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]],
            ['super_kernel'])
        self.assertEqual(userdata["pkgcb_calls"], 1)
        self.assertEqual(userdata["warnings"], [])

        pkg = userdata["pkgs"][0]
        self.assertEqual(pkg.pkgId, "152824bff2aa6d54f429d43e87a3ff3a0286505c6d93ec87692b5e3a9e3b97bf")
        self.assertEqual(pkg.name, "super_kernel")
        self.assertEqual(pkg.arch, "x86_64")
        self.assertEqual(pkg.version, "6.0.1")
        self.assertEqual(pkg.epoch, "0")
        self.assertEqual(pkg.release, "2")
        self.assertEqual(pkg.summary, None)
        self.assertEqual(pkg.description, None)
        self.assertEqual(pkg.url, None)
        self.assertEqual(pkg.time_file, 0)
        self.assertEqual(pkg.time_build, 0)
        self.assertEqual(pkg.rpm_license, None)
        self.assertEqual(pkg.rpm_vendor, None)
        self.assertEqual(pkg.rpm_group, None)
        self.assertEqual(pkg.rpm_buildhost, None)
        self.assertEqual(pkg.rpm_sourcerpm, None)
        self.assertEqual(pkg.rpm_header_start, 0)
        self.assertEqual(pkg.rpm_header_end, 0)
        self.assertEqual(pkg.rpm_packager, None)
        self.assertEqual(pkg.size_package, 0)
        self.assertEqual(pkg.size_installed, 0)
        self.assertEqual(pkg.size_archive, 0)
        self.assertEqual(pkg.location_href, None)
        self.assertEqual(pkg.location_base, None)
        self.assertEqual(pkg.checksum_type, None)
        self.assertEqual(pkg.requires, [])
        self.assertEqual(pkg.provides, [])
        self.assertEqual(pkg.conflicts, [])
        self.assertEqual(pkg.obsoletes, [])
        self.assertEqual(pkg.files,
                [(None, '/usr/bin/', 'super_kernel'),
                 (None, '/usr/share/man/', 'super_kernel.8.gz')])
        self.assertEqual(pkg.changelogs, [])

    def test_xml_parser_filelists_repo02(self):

        userdata = {
                "pkgs": [],
                "pkgcb_calls": 0,
                "warnings": []
            }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        def pkgcb(pkg):
            userdata["pkgcb_calls"] += 1

        def warningcb(warn_type, msg):
            userdata["warnings"].append((warn_type, msg))

        cr.xml_parse_filelists(REPO_02_FILXML, newpkgcb, pkgcb, warningcb)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]],
            ['fake_bash', 'super_kernel'])
        self.assertEqual(userdata["pkgcb_calls"], 2)
        self.assertEqual(userdata["warnings"], [])

    def test_xml_parser_filelists_repo02_only_pkgcb(self):

        pkgs = []

        def pkgcb(pkg):
            pkgs.append(pkg)

        cr.xml_parse_filelists(REPO_02_FILXML, None, pkgcb, None)

        self.assertEqual([pkg.name for pkg in pkgs],
            ['fake_bash', 'super_kernel'])

    def test_xml_parser_filelists_repo02_no_cbs(self):
        self.assertRaises(ValueError,
                          cr.xml_parse_filelists,
                          REPO_02_FILXML, None, None, None)

    def test_xml_parser_filelists_warnings(self):

        userdata = {
                "pkgs": [],
                "warnings": []
            }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        def warningcb(warn_type, msg):
            userdata["warnings"].append((warn_type, msg))

        cr.xml_parse_filelists(FILELISTS_MULTI_WARN_00_PATH,
                               newpkgcb,
                               None,
                               warningcb)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]],
            ['fake_bash', 'super_kernel'])
        self.assertEqual(userdata["warnings"],
            [(1, 'Missing attribute "arch" of a package element'),
             (2, 'Unknown file type "xxx"'),
             (0, 'Unknown element "bar"')])

    def test_xml_parser_filelists_error(self):

        userdata = { "pkgs": [] }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_filelists,
                          FILELISTS_ERROR_00_PATH, newpkgcb, None, None)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]], [])

    def test_xml_parser_filelists_newpkgcb_abort(self):
        def newpkgcb(pkgId, name, arch):
            raise Error("Foo error")
        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_filelists,
                          REPO_02_FILXML, newpkgcb, None, None)

    def test_xml_parser_filelists_pkgcb_abort(self):
        def newpkgcb(pkgId, name, arch):
            return cr.Package()
        def pkgcb(pkg):
            raise Error("Foo error")
        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_filelists,
                          REPO_02_FILXML, newpkgcb, pkgcb, None)

    def test_xml_parser_filelists_warningcb_abort(self):
        def newpkgcb(pkgId, name, arch):
            return cr.Package()
        def warningcb(type, msg):
            raise Error("Foo error")
        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_filelists,
                          FILELISTS_MULTI_WARN_00_PATH,
                          newpkgcb, None, warningcb)

class TestCaseXmlParserOther(unittest.TestCase):

    def test_xml_parser_other_repo01(self):

        userdata = {
                "pkgs": [],
                "pkgcb_calls": 0,
                "warnings": []
            }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        def pkgcb(pkg):
            userdata["pkgcb_calls"] += 1

        def warningcb(warn_type, msg):
            userdata["warnings"].append((warn_type, msg))

        cr.xml_parse_other(REPO_01_OTHXML, newpkgcb, pkgcb, warningcb)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]],
            ['super_kernel'])
        self.assertEqual(userdata["pkgcb_calls"], 1)
        self.assertEqual(userdata["warnings"], [])

        pkg = userdata["pkgs"][0]
        self.assertEqual(pkg.pkgId, "152824bff2aa6d54f429d43e87a3ff3a0286505c6d93ec87692b5e3a9e3b97bf")
        self.assertEqual(pkg.name, "super_kernel")
        self.assertEqual(pkg.arch, "x86_64")
        self.assertEqual(pkg.version, "6.0.1")
        self.assertEqual(pkg.epoch, "0")
        self.assertEqual(pkg.release, "2")
        self.assertEqual(pkg.summary, None)
        self.assertEqual(pkg.description, None)
        self.assertEqual(pkg.url, None)
        self.assertEqual(pkg.time_file, 0)
        self.assertEqual(pkg.time_build, 0)
        self.assertEqual(pkg.rpm_license, None)
        self.assertEqual(pkg.rpm_vendor, None)
        self.assertEqual(pkg.rpm_group, None)
        self.assertEqual(pkg.rpm_buildhost, None)
        self.assertEqual(pkg.rpm_sourcerpm, None)
        self.assertEqual(pkg.rpm_header_start, 0)
        self.assertEqual(pkg.rpm_header_end, 0)
        self.assertEqual(pkg.rpm_packager, None)
        self.assertEqual(pkg.size_package, 0)
        self.assertEqual(pkg.size_installed, 0)
        self.assertEqual(pkg.size_archive, 0)
        self.assertEqual(pkg.location_href, None)
        self.assertEqual(pkg.location_base, None)
        self.assertEqual(pkg.checksum_type, None)
        self.assertEqual(pkg.requires, [])
        self.assertEqual(pkg.provides, [])
        self.assertEqual(pkg.conflicts, [])
        self.assertEqual(pkg.obsoletes, [])
        self.assertEqual(pkg.files, [])
        self.assertEqual(pkg.changelogs,
                [('Tomas Mlcoch <tmlcoch@redhat.com> - 6.0.1-1',
                   1334664000,
                  '- First release'),
                 ('Tomas Mlcoch <tmlcoch@redhat.com> - 6.0.1-2',
                   1334664001,
                   '- Second release')])

    def test_xml_parser_other_repo02(self):

        userdata = {
                "pkgs": [],
                "pkgcb_calls": 0,
                "warnings": []
            }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        def pkgcb(pkg):
            userdata["pkgcb_calls"] += 1

        def warningcb(warn_type, msg):
            userdata["warnings"].append((warn_type, msg))

        cr.xml_parse_other(REPO_02_OTHXML, newpkgcb, pkgcb, warningcb)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]],
            ['fake_bash', 'super_kernel'])
        self.assertEqual(userdata["pkgcb_calls"], 2)
        self.assertEqual(userdata["warnings"], [])

    def test_xml_parser_other_repo02_only_pkgcb(self):

        pkgs = []

        def pkgcb(pkg):
            pkgs.append(pkg)

        cr.xml_parse_other(REPO_02_OTHXML, None, pkgcb, None)

        self.assertEqual([pkg.name for pkg in pkgs],
            ['fake_bash', 'super_kernel'])

    def test_xml_parser_other_repo02_no_cbs(self):
        self.assertRaises(ValueError,
                          cr.xml_parse_other,
                          REPO_02_OTHXML, None, None, None)

    def test_xml_parser_other_warnings(self):

        userdata = {
                "pkgs": [],
                "warnings": []
            }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        def warningcb(warn_type, msg):
            userdata["warnings"].append((warn_type, msg))

        cr.xml_parse_other(OTHER_MULTI_WARN_00_PATH,
                           newpkgcb,
                           None,
                           warningcb)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]],
            [None, 'super_kernel'])
        self.assertEqual(userdata["warnings"],
            [(1, 'Missing attribute "name" of a package element'),
             (0, 'Unknown element "bar"'),
             (3, 'Conversion of "xxx" to integer failed')])

    def test_xml_parser_other_error(self):

        userdata = { "pkgs": [] }

        def newpkgcb(pkgId, name, arch):
            pkg = cr.Package()
            userdata["pkgs"].append(pkg)
            return pkg

        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_other,
                          OTHER_ERROR_00_PATH, newpkgcb, None, None)

        self.assertEqual([pkg.name for pkg in userdata["pkgs"]], [])

    def test_xml_parser_other_newpkgcb_abort(self):
        def newpkgcb(pkgId, name, arch):
            raise Error("Foo error")
        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_other,
                          REPO_02_OTHXML, newpkgcb, None, None)

    def test_xml_parser_other_pkgcb_abort(self):
        def newpkgcb(pkgId, name, arch):
            return cr.Package()
        def pkgcb(pkg):
            raise Error("Foo error")
        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_other,
                          REPO_02_OTHXML, newpkgcb, pkgcb, None)

    def test_xml_parser_other_warningcb_abort(self):
        def newpkgcb(pkgId, name, arch):
            return cr.Package()
        def warningcb(type, msg):
            raise Error("Foo error")
        self.assertRaises(cr.CreaterepoCError,
                          cr.xml_parse_other,
                          OTHER_MULTI_WARN_00_PATH,
                          newpkgcb, None, warningcb)

class TestCaseXmlParserRepomd(unittest.TestCase):

    def test_xml_parser_repomd_bad_repomd_object(self):
        self.assertRaises(TypeError,
                          cr.xml_parse_repomd,
                          REPO_01_REPOMD, "foo", None)

    def test_xml_parser_repomd_repo01(self):

        warnings = []

        def warningcb(warn_type, msg):
            warnings.append((warn_type, msg))

        repomd = cr.Repomd()

        cr.xml_parse_repomd(REPO_01_REPOMD, repomd, warningcb)

        self.assertEqual(warnings, [])

        self.assertEqual(repomd.revision, "1334667230")
        self.assertEqual(repomd.repo_tags, [])
        self.assertEqual(repomd.distro_tags, [])
        self.assertEqual(repomd.content_tags, [])
        self.assertEqual(len(repomd.records), 3)

        self.assertEqual(repomd.records[0].type, "filelists")
        self.assertEqual(repomd.records[0].location_real, None)
        self.assertEqual(repomd.records[0].location_href,
            "repodata/c7db035d0e6f1b2e883a7fa3229e2d2be70c05a8b8d2b57dbb5f9c1a67483b6c-filelists.xml.gz")
        self.assertEqual(repomd.records[0].checksum,
            "c7db035d0e6f1b2e883a7fa3229e2d2be70c05a8b8d2b57dbb5f9c1a67483b6c")
        self.assertEqual(repomd.records[0].checksum_type, "sha256")
        self.assertEqual(repomd.records[0].checksum_open,
            "85bc611be5d81ac8da2fe01e98ef741d243d1518fcc46ada70660020803fbf09")
        self.assertEqual(repomd.records[0].checksum_open_type, "sha256")
        self.assertEqual(repomd.records[0].timestamp, 1334667230)
        self.assertEqual(repomd.records[0].size, 273)
        self.assertEqual(repomd.records[0].size_open, 389)
        self.assertEqual(repomd.records[0].db_ver, 0)

        self.assertEqual(repomd.records[1].type, "other")
        self.assertEqual(repomd.records[1].location_real, None)
        self.assertEqual(repomd.records[1].location_href,
            "repodata/b752a73d9efd4006d740f943db5fb7c2dd77a8324bd99da92e86bd55a2c126ef-other.xml.gz")
        self.assertEqual(repomd.records[1].checksum,
            "b752a73d9efd4006d740f943db5fb7c2dd77a8324bd99da92e86bd55a2c126ef")
        self.assertEqual(repomd.records[1].checksum_type, "sha256")
        self.assertEqual(repomd.records[1].checksum_open,
            "da6096c924349af0c326224a33be0cdb26897fbe3d25477ac217261652449445")
        self.assertEqual(repomd.records[1].checksum_open_type, "sha256")
        self.assertEqual(repomd.records[1].timestamp, 1334667230)
        self.assertEqual(repomd.records[1].size, 332)
        self.assertEqual(repomd.records[1].size_open, 530)
        self.assertEqual(repomd.records[1].db_ver, 0)

        self.assertEqual(repomd.records[2].type, "primary")
        self.assertEqual(repomd.records[2].location_real, None)
        self.assertEqual(repomd.records[2].location_href,
            "repodata/6c662d665c24de9a0f62c17d8fa50622307739d7376f0d19097ca96c6d7f5e3e-primary.xml.gz")
        self.assertEqual(repomd.records[2].checksum,
            "6c662d665c24de9a0f62c17d8fa50622307739d7376f0d19097ca96c6d7f5e3e")
        self.assertEqual(repomd.records[2].checksum_type, "sha256")
        self.assertEqual(repomd.records[2].checksum_open,
            "0fc6cadf97d515e87491d24dc9712d8ddaf2226a21ae7f131ff42d71a877c496")
        self.assertEqual(repomd.records[2].checksum_open_type, "sha256")
        self.assertEqual(repomd.records[2].timestamp, 1334667230)
        self.assertEqual(repomd.records[2].size, 782)
        self.assertEqual(repomd.records[2].size_open, 2085)
        self.assertEqual(repomd.records[2].db_ver, 0)

    def test_xml_parser_repomd_repo01_nowarningcb(self):

        repomd = cr.Repomd()
        cr.xml_parse_repomd(REPO_01_REPOMD, repomd)
        self.assertEqual(repomd.revision, "1334667230")
        self.assertEqual(repomd.repo_tags, [])
        self.assertEqual(repomd.distro_tags, [])
        self.assertEqual(repomd.content_tags, [])
        self.assertEqual(len(repomd.records), 3)




#!/usr/bin/env python

import os
import os.path

import createrepo_c as cr

REPO_PATH = "repo/"

def parse_repomd(path):
    repomd = cr.Repomd(path)
    print "Revision:", repomd.revision
    if repomd.contenthash:
        print "Contenthash:", repomd.contenthash
        print "Contenthash type:", repomd.contenthash_type
    print "Repo tags:", repomd.repo_tags
    print "Content tags:", repomd.content_tags
    print "Distro tags:", repomd.distro_tags
    print
    for rec in repomd.records:
        print "Type:", rec.type
        print "Location href:", rec.location_href
        print "Location base:", rec.location_base
        print "Checksum:", rec.checksum
        print "Checksum type:", rec.checksum_type
        print "Checksum open:", rec.checksum_open
        print "Checksum open type:", rec.checksum_open_type
        print "Timestamp:", rec.timestamp
        print "Size:", rec.size
        print "Size open:", rec.size_open
        if rec.db_ver:
            print "Db version:", rec.db_ver
        print

if __name__ == "__main__":
    repomd_path = os.path.join(REPO_PATH, "repodata/repomd.xml")
    parse_repomd(repomd_path)



import unittest
import createrepo_c as cr

from . import fixtures

class TestCaseVersion(unittest.TestCase):
    def test_version(self):
        self.assertTrue(isinstance(cr.VERSION_MAJOR, int));
        self.assertTrue(isinstance(cr.VERSION_MINOR, int));
        self.assertTrue(isinstance(cr.VERSION_PATCH, int));



from skbuild import setup


with open('VERSION.cmake', 'r+') as version_file:
    lines = version_file.read().splitlines()
    # parse out digit characters from the line, convert to int
    numbers = [int("".join(filter(str.isdigit, line))) for line in lines]
    # build version string
    version = '{major}.{minor}.{patch}'.format(
        major=numbers[0],
        minor=numbers[1],
        patch=numbers[2]
    )

setup(
    name='createrepo_c',
    description='C implementation of createrepo',
    version=version,
    license='GPLv2+',
    author='RPM Software Management',
    author_email='rpm-ecosystem@lists.rpm.org',
    url='https://github.com/rpm-software-management',
    classifiers=(
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: C',
        'Topic :: System :: Software Distribution',
        'Topic :: System :: Systems Administration',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ),
    packages=['createrepo_c'],
    package_dir={
        'createrepo_c': 'src/python/createrepo_c'
    },
    cmake_args=[
        '-DBIN_INSTALL_DIR:PATH=src/python/createrepo_c/data/bin',
        '-DBUILD_LIBCREATEREPO_C_SHARED:BOOL=OFF',
        '-DCREATEREPO_C_INSTALL_DEVELOPMENT:BOOL=OFF',
        '-DCREATEREPO_C_INSTALL_MANPAGES:BOOL=OFF',
        '-DENABLE_BASHCOMP:BOOL=OFF',
        '-DENABLE_DRPM:BOOL=OFF'
    ],
    cmake_languages=['C'],
    entry_points={
        'console_scripts': [
            'createrepo_c=createrepo_c:createrepo_c',
            'mergerepo_c=createrepo_c:mergerepo_c',
            'modifyrepo_c=createrepo_c:modifyrepo_c',
            'sqliterepo_c=createrepo_c:sqliterepo_c'
        ]
    },
)



import os
import re
import time
import shutil
import pprint
import gzip
import filecmp
import os.path
import tempfile
import unittest
import threading
import subprocess

from .fixtures import PACKAGESDIR

OUTPUTDIR = None
OUTPUTDIR_LOCK = threading.Lock()

def get_outputdir():
    global OUTPUTDIR
    if OUTPUTDIR:
        return OUTPUTDIR
    OUTPUTDIR_LOCK.acquire()
    if not OUTPUTDIR:
        prefix = time.strftime("./testresults_%Y%m%d_%H%M%S_")
        OUTPUTDIR = tempfile.mkdtemp(prefix=prefix, dir="./")
    OUTPUTDIR_LOCK.release()
    return OUTPUTDIR


class _Result(object):
    def __str__(self):
        return pprint.pformat(self.__dict__)


class CrResult(_Result):
    def __init__(self):
        self.rc = None       # Return code
        self.out = None      # stdout + stderr
        self.dir = None      # Directory that was processed
        self.prog = None     # Program name
        self.cmd = None      # Complete command
        self.outdir = None   # Output directory
        self.logfile = None  # Log file where the out was logged


class RepoDiffResult(_Result):
    def __init__(self):
        self.rc = None
        self.out = None
        self.repo1 = None
        self.repo2 = None
        self.cmd = None      # Complete command
        self.logfile = None  # Log file where the out was logged


class RepoSanityCheckResult(_Result):
    def __init__(self):
        self.rc = None
        self.out = None
        self.repo = None
        self.cmd = None
        self.logfile = None


class BaseTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        self.outdir = get_outputdir()
        self.tcdir = os.path.join(self.outdir, self.__class__.__name__)
        if not os.path.exists(self.tcdir):
            os.mkdir(self.tcdir)
        if self.__class__.__doc__:
            description_fn = os.path.join(self.tcdir, "description")
            open(description_fn, "w").write(self.__class__.__doc__+'\n')
        unittest.TestCase.__init__(self, *args, **kwargs)

        self.tdir = None            # Test dir for the current test
        self.indir = None           # Input dir for the current test
        self._currentResult = None  # Result of the current test

        # Prevent use of a first line from test docstring as its name in output
        self.shortDescription_orig = self.shortDescription
        self.shortDescription = self._shortDescription
        self.main_cwd = os.getcwd()

    def _shortDescription(self):
        return ".".join(self.id().split('.')[-2:])

    def run(self, result=None):
        # Hacky
        self._currentResult = result # remember result for use in tearDown
        unittest.TestCase.run(self, result)

    def setUp(self):
        os.chdir(self.main_cwd) # In case of TimedOutException in Nose test... the tearDown is not called :-/
        caller = self.id().split(".", 3)[-1]
        self.tdir = os.path.abspath(os.path.join(self.tcdir, caller))
        os.mkdir(self.tdir)
        self.indir = os.path.join(self.tdir, "input")
        os.mkdir(self.indir)
        description = self.shortDescription_orig()
        if description:
            fn = os.path.join(self.tdir, "description")
            open(fn, "w").write(description+'\n')
        #self.log = # TODO
        os.chdir(self.tdir)
        self.setup()

    def setup(self):
        pass

    def tearDown(self):
        if self.tdir and self._currentResult:
            if not len(self._currentResult.errors) + len(self._currentResult.failures):
                self.set_success()
        self.teardown()
        os.chdir(self.main_cwd)

    def teardown(self):
        pass

    def runcmd(self, cmd, logfile=None, workdir=None, stdin_data=None):
        """Stolen from the kobo library.
         Author of the original function is dmach@redhat.com"""

        # TODO: Add time how long the command takes

        if type(cmd) in (list, tuple):
            import pipes
            cmd = " ".join(pipes.quote(i) for i in cmd)

        if logfile is not None:
            already_exists = False
            if os.path.exists(logfile):
                already_exists = True
            logfile = open(logfile, "a")
            if already_exists:
                logfile.write("\n{0}\n Another run\n{0}\n".format('='*79))
            logfile.write("cd %s\n" % os.getcwd())
            for var in ("PATH", "PYTHONPATH", "LD_LIBRARY_PATH"):
                logfile.write('export %s="%s"\n' % (var, os.environ.get(var,"")))
            logfile.write("\n")
            logfile.write(cmd+"\n")
            logfile.write("\n+%s+\n\n" % ('-'*77))

        stdin = None
        if stdin_data is not None:
            stdin = subprocess.PIPE

        proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT, cwd=workdir)

        if stdin_data is not None:
            class StdinThread(threading.Thread):
                def run(self):
                    proc.stdin.write(stdin_data)
                    proc.stdin.close()
            stdin_thread = StdinThread()
            stdin_thread.daemon = True
            stdin_thread.start()

        output = ""
        while True:
            lines = proc.stdout.readline().decode('utf-8')
            if lines == "":
                break
            if logfile:
                logfile.write(lines)
            output += lines
        proc.wait()

        if logfile:
            logfile.write("\n+%s+\n\n" % ('-'*77))
            logfile.write("RC: %s (%s)\n" % (proc.returncode, "Success" if not proc.returncode else "Failure"))
            logfile.close()

        if stdin_data is not None:
            stdin_thread.join()

        return proc.returncode, output

    def set_success(self):
        """Create a SUCCESS file in directory of the current test"""
        fn = os.path.join(self.tdir, "SUCCESS")
        open(fn, "w")

    def copy_pkg(self, name, dst):
        """Copy package from testdata into specified destination"""
        src = os.path.join(PACKAGESDIR, name)
        shutil.copy(src, dst)
        return os.path.join(dst, name)

    def indir_addpkg(self, name):
        """Add package into input dir for the current test"""
        src = os.path.join(PACKAGESDIR, name)
        return self.copy_pkg(src, self.indir)

    def indir_makedirs(self, path):
        """Make a directory in input dir of the current test"""
        path = path.lstrip('/')
        final_path = os.path.join(self.indir, path)
        os.makedirs(final_path)
        return final_path

    def indir_mkfile(self, name, content=""):
        """Create a file in input dir of the current test"""
        fn = os.path.join(self.indir, name)
        with open(fn, "w") as f:
            f.write(content)
        return fn

    def tdir_makedirs(self, path):
        """Make a directory in test dir of the current test"""
        path = path.lstrip('/')
        final_path = os.path.join(self.tdir, path)
        os.makedirs(final_path)
        return final_path

    def run_prog(self, prog, dir, args=None, outdir=None):
        res = CrResult()
        res.dir = dir
        res.prog = prog
        res.outdir = outdir
        res.logfile =  os.path.join(self.tdir, "out_%s" % res.prog)
        res.cmd = "%(prog)s --verbose %(args)s %(dir)s" % {
            "prog": res.prog,
            "dir": res.dir,
            "args": args or "",
        }
        res.rc, res.out = self.runcmd(res.cmd, logfile=res.logfile)
        return res

    def run_cr(self, dir, args=None, c=False, outdir=None):
        """Run createrepo and return CrResult object with results

        :returns: Result of the createrepo run
        :rtype: CrResult
        """
        prog = "createrepo_c" if c else "createrepo"

        if not outdir:
            outdir = os.path.join(self.tdir, prog)
        else:
            outdir = os.path.join(self.tdir, outdir)
        if not os.path.exists(outdir):
            os.mkdir(outdir)

        args = " -o %s %s" % (outdir, args if args else "")

        res = self.run_prog(prog, dir, args, outdir)
        return res

    def run_sqlr(self, dir, args=None):
        """"""
        res = self.run_prog("sqliterepo_c", dir, args)
        return res

    def compare_repos(self, repo1, repo2):
        """Compare two repos

        :returns: Difference between two repositories
        :rtype: RepoDiffResult
        """
        res = RepoDiffResult()
        res.repo1 = os.path.join(repo1, "repodata")
        res.repo2 = os.path.join(repo2, "repodata")
        res.logfile = os.path.join(self.tdir, "out_cmp")
        res.cmd = "yum-metadata-diff --verbose --compare %s %s" % (res.repo1, res.repo2)
        res.rc, res.out = self.runcmd(res.cmd, logfile=res.logfile)
        return res

    def check_repo_sanity(self, repo):
        """Check if a repository is sane

        :returns: Result of the sanity check
        :rtype: RepoSanityCheckResult
        """
        res = RepoSanityCheckResult()
        res.repo = os.path.join(repo, "repodata")
        res.logfile = os.path.join(self.tdir, "out_sanity")
        res.cmd = "yum-metadata-diff --verbose --check %s" % res.repo
        res.rc, res.out = self.runcmd(res.cmd, logfile=res.logfile)
        return res

    def assert_run_cr(self, *args, **kwargs):
        """Run createrepo and assert that it finished with return code 0

        :returns: Result of the createrepo run
        :rtype: CrResult
        """
        res = self.run_cr(*args, **kwargs)
        self.assertFalse(res.rc)
        return res

    def assert_run_sqlr(self, *args, **kwargs):
        """Run sqliterepo and assert that it finished with return code 0

        :returns: Result of the sqliterepo run
        :rtype: CrResult
        """
        res = self.run_sqlr(*args, **kwargs)
        self.assertFalse(res.rc)
        return res

    def assert_same_results(self, indir, args=None):
        """Run both createrepo and createrepo_c and assert that results are same

        :returns: (result of comparison, createrepo result, createrepo_c result)
        :rtype: (RepoDiffResult, CrResult, CrResult)
        """
        crres = self.run_cr(indir, args)
        crcres = self.run_cr(indir, args, c=True)
        self.assertFalse(crres.rc)   # Error while running createrepo
        self.assertFalse(crcres.rc)  # Error while running createrepo_c
        cmpres = self.compare_repos(crres.outdir, crcres.outdir)
        self.assertFalse(cmpres.rc)  # Repos are not same
        return (cmpres, crres, crcres)

    def assert_repo_sanity(self, repo):
        """Run repo sanity check and assert it's sane

        :returns: Result of the sanity check
        :rtype: RepoSanityCheckResult
        """
        res = self.check_repo_sanity(repo)
        self.assertFalse(res.rc)
        return res

    def assert_repo_files(self, repo, file_patterns, additional_files_allowed=True):
        """Assert that files (defined by re) are in the repo
        """
        compiled_patterns = map(re.compile, file_patterns)
        fns = os.listdir(os.path.join(repo, "repodata/"))
        used_patterns = []
        for pattern in compiled_patterns:
            for fn in fns[:]:
                if pattern.match(fn):
                    fns.remove(fn)
                    used_patterns.append(pattern)

        if not additional_files_allowed:
            self.assertEqual(fns, [])  # Unexpected additional files

        unused_paterns = [x.pattern for x in (set(compiled_patterns) - set(used_patterns))]
        self.assertEqual(unused_paterns, [])  # Some patterns weren't used

    def assert_same_dir_content(self, a, b):
        """Assert identical content of two directories
        (Not recursive yet)
        """
        # TODO: Recursive
        self.assertTrue(os.path.isdir(a))
        self.assertTrue(os.path.isdir(b))

        _, logfn = tempfile.mkstemp(prefix="out_dircmp_%s_" % long(time.time()), dir=self.tdir)
        logfile = open(logfn, "w")
        logfile.write("A: %s\n" % a)
        logfile.write("B: %s\n" % b)
        logfile.write("\n")

        dircmpobj = filecmp.dircmp(a, b)
        if dircmpobj.left_only or dircmpobj.right_only or dircmpobj.diff_files:
            logfile.write("A only:\n%s\n\n" % dircmpobj.left_only)
            logfile.write("B only:\n%s\n\n" % dircmpobj.right_only)
            logfile.write("Diff files:\n%s\n\n" % dircmpobj.diff_files)
            logfile.close()
            self.assertTrue(False)
        logfile.write("OK\n")
        logfile.close()

    def assert_file_contains_line(self, path_to_file, line):
        contents = ""
        found = False

        if path_to_file.endswith(".gz"):
            with gzip.open(path_to_file, 'rb') as f:
                for l in f.readlines():
                    if (l.strip() == line.strip()):
                        found = True
                self.assertTrue(found, "Line: " + line + " not found in file: " + path_to_file)
                return

        with open(path_to_file) as f:
            for l in f.readlines():
                if (l.strip() == line.strip()):
                    found = True

        self.assertTrue(found, "Line: " + line + " not found in file: " + path_to_file)
